@@ -20,7 +20,7 @@ libfontconfig-1.dll libfreetype-6.dll libgdk_pixbuf-2.0-0.dll
 libgdk-3-0.dll libgio-2.0-0.dll libglib-2.0-0.dll libgmodule-2.0-0.dll
 libgobject-2.0-0.dll libgraphite2.dll libgsl-23.dll libgslcblas-0.dll
 libgtk-3-0.dll libharfbuzz-0.dll libiconv-2.dll libintl-8.dll
-libobjc-4.dll libp11-kit-0.dll libpango-1.0-0.dll
+libobjc-4.dll libp11-kit-0.dll libpango-1.0-0.dll libfribidi-0.dll
 libgtksourceview-3.0-1.dll libpangocairo-1.0-0.dll
 libpangoft2-1.0-0.dll libpangowin32-1.0-0.dll libpcre-1.dll
 libpixman-1-0.dll libpng16-16.dll libstdc++-6.dll libtermcap-0.dll
