@@ -22,7 +22,7 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 
-#ifdef __MINGW32__
+#ifdef G_OS_WIN32
 #define WINDOWS 1
 #ifndef _WIN32_WINNT
 #define _WIN32_WINNT 0x0500
@@ -38,6 +38,11 @@
 #define OSX 0
 #endif
 
+#ifdef __WIN64__
+#define WINDOWS 1
+#include <windows.h>
+#endif
+
 #ifndef CCNAME
 #define CCNAME "unknown";
 #endif
