@@ -141,6 +141,36 @@ static int GRAinview(int GC,int x,int y);
 static int GRArectclip(int GC,int *x0,int *y0,int *x1,int *y1);
 static int GRAlineclip(int GC,int *x0,int *y0,int *x1,int *y1);
 
+int
+calc_zoom_direction(int direction, double zx, double zy, double *zp, double *zn)
+{
+  double x, y, dir;
+  int new_dir;
+  dir = (direction / 100.0) / 180 * MPI;
+
+  if (zn) {
+    x = sin(dir) * zx;
+    y = cos(dir) * zy;
+    *zn = sqrt(x * x + y * y);
+  }
+
+  x = cos(dir) * zx;
+  y = sin(dir) * zy;
+  if (zp) {
+    *zp = sqrt(x * x + y * y);
+  }
+
+  if (x == 0) {
+    if (y > 0) {
+      new_dir = 9000;
+    } else {
+      new_dir = 27000;
+    }
+  } else {
+    new_dir = atan(y / x) / MPI * 18000;
+  }
+  return new_dir;
+}
 
 int
 _GRAopencallback(directfunc direct,struct narray **list,void *local)
@@ -1188,12 +1218,46 @@ GRAlines(int GC,int num,int *point)
 }
 
 void
-GRAmark(int GC,int type,int x0,int y0,int size,
+GRArotate(int x0, int y0, int *pos, int *rpos, int n, double dx, double dy)
+{
+  int i;
+  double x, y;
+  for (i = 0; i < n; i++) {
+    x = pos[i * 2] - x0;
+    y = pos[i * 2 + 1] - y0;
+    rpos[i * 2] = nround(dx * x - dy * y) + x0;
+    rpos[i * 2 + 1] = nround(dy * x + dx * y) + y0;
+  }
+}
+
+void
+GRArectangle_rotate(int GC,double dx, double dy, int cx, int cy, int x0,int y0,int x1,int y1,int fil)
+{
+  int po[8];
+
+  if (dy == 0) {
+    GRArectangle(GC, x0, y0, x1, y1, fil);
+    return;
+  }
+  po[0]=x0;
+  po[1]=y0;
+  po[2]=x0;
+  po[3]=y1;
+  po[4]=x1;
+  po[5]=y1;
+  po[6]=x1;
+  po[7]=y0;
+  GRArotate(cx, cy, po, po, 4, dx, dy);
+  GRAdrawpoly(GC,4,po,fil);
+}
+
+void
+GRAmark_rotate(int GC,int type,int x0,int y0, double dx, double dy, int size,
 	int fr,int fg,int fb, int fa, int br,int bg,int bb, int ba)
 {
   int x1,y1,x2,y2,r;
-  int po[10],po2[10];
-  int type2,sgn;
+  int po[12],po2[12],rpo[12],rpo2[12];
+  int type2,sgn, ofst;
   double d;
 
   if ((GRAClist[GC].clip==0) || (GRAinview(GC,x0,y0)==0)) {
@@ -1223,17 +1287,28 @@ GRAmark(int GC,int type,int x0,int y0,int size,
           r/=2;
           GRAcircle(GC,x0,y0,r,r,0,36000,0);
         } else if (type!=1) {
+	  if (dy == 0) {
+	    if (dx >= 0) {
+	      ofst = 0;
+	    } else {
+	      ofst = 18000;
+	    }
+	  } else if (dy > 0) {
+	    ofst = acos(-dx) / MPI * 18000 + 18000;
+	  } else {
+	    ofst = acos(dx) / MPI * 18000;
+	  }
           if (type2==4) {
             r/=2;
             GRAcircle(GC,x0,y0,r,r,0,36000,1);
           } else if (type2==6) {
-            GRAcircle(GC,x0,y0,r,r,27000,18000,1);
+            GRAcircle(GC,x0,y0,r,r,27000 + ofst,18000,1);
           } else if (type2==7) {
-            GRAcircle(GC,x0,y0,r,r,9000,18000,1);
+            GRAcircle(GC,x0,y0,r,r,9000 + ofst, 18000,1);
           } else if (type2==8) {
-            GRAcircle(GC,x0,y0,r,r,0,18000,1);
+            GRAcircle(GC,x0,y0,r,r,ofst,18000,1);
           } else if (type2==9) {
-            GRAcircle(GC,x0,y0,r,r,18000,18000,1);
+            GRAcircle(GC,x0,y0,r,r,18000 + ofst, 18000,1);
 	  }
 	}
       }
@@ -1247,51 +1322,51 @@ GRAmark(int GC,int type,int x0,int y0,int size,
       y2=y0+size/2;
       if (type2==0) {
         GRAcolor(GC,fr,fg,fb, fa);
-        GRArectangle(GC,x1,y1,x2,y2,1);
+        GRArectangle_rotate(GC,dx,dy,x0,y0,x1,y1,x2,y2,1);
       } else if (type2==2) {
         GRAcolor(GC,fr,fg,fb, fa);
-        GRArectangle(GC,x1,y1,x2,y2,0);
+        GRArectangle_rotate(GC,dx,dy,x0,y0,x1,y1,x2,y2,0);
       } else if (type2==5) {
         GRAcolor(GC,fr,fg,fb, fa);
-        GRArectangle(GC,x1,y1,x2,y2,1);
+        GRArectangle_rotate(GC,dx,dy,x0,y0,x1,y1,x2,y2,1);
         x1=x0-size/4;
         y1=y0-size/4;
         x2=x0+size/4;
         y2=y0+size/4;
         GRAcolor(GC,br,bg,bb, ba);
-        GRArectangle(GC,x1,y1,x2,y2,1);
+        GRArectangle_rotate(GC,dx,dy,x0,y0,x1,y1,x2,y2,1);
       } else {
         GRAcolor(GC,br,bg,bb, ba);
-        GRArectangle(GC,x1,y1,x2,y2,1);
+        GRArectangle_rotate(GC,dx,dy,x0,y0,x1,y1,x2,y2,1);
         GRAcolor(GC,fr,fg,fb, fa);
-        GRArectangle(GC,x1,y1,x2,y2,0);
+        GRArectangle_rotate(GC,dx,dy,x0,y0,x1,y1,x2,y2,0);
         if (type2==3) {
           x1=x0-size/4;
           y1=y0-size/4;
           x2=x0+size/4;
           y2=y0+size/4;
-          GRArectangle(GC,x1,y1,x2,y2,0);
+          GRArectangle_rotate(GC,dx,dy,x0,y0,x1,y1,x2,y2,0);
         } else if (type2!=1) {
           if (type2==4) {
             x1=x0-size/4;
             y1=y0-size/4;
             x2=x0+size/4;
             y2=y0+size/4;
-            GRArectangle(GC,x1,y1,x2,y2,1);
+            GRArectangle_rotate(GC,dx,dy,x0,y0,x1,y1,x2,y2,1);
           } else if (type2==6) {
             x1=x0;
-            GRArectangle(GC,x1,y1,x2,y2,1);
+            GRArectangle_rotate(GC,dx,dy,x0,y0,x1,y1,x2,y2,1);
           } else if (type2==7) {
             x2=x0;
-            GRArectangle(GC,x1,y1,x2,y2,1);
+            GRArectangle_rotate(GC,dx,dy,x0,y0,x1,y1,x2,y2,1);
           } else if (type2==8) {
             y2=y0;
-            GRArectangle(GC,x1,y1,x2,y2,1);
+            GRArectangle_rotate(GC,dx,dy,x0,y0,x1,y1,x2,y2,1);
           } else if (type2==9) {
             y1=y0;
-            GRArectangle(GC,x1,y1,x2,y2,1);
-	      }
-	    }
+            GRArectangle_rotate(GC,dx,dy,x0,y0,x1,y1,x2,y2,1);
+	  }
+	}
       }
       break;
     case 20: case 21: case 22: case 23: case 24:
@@ -1307,34 +1382,37 @@ GRAmark(int GC,int type,int x0,int y0,int size,
       po[7]=y0;
       po[8]=x0;
       po[9]=y0-size/2;
+      GRArotate(x0, y0, po, rpo, 5, dx, dy);
       if (type2==0) {
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,5,po,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,5,rpo,GRA_FILL_MODE_EVEN_ODD);
       } else if (type2==2) {
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,5,po,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,5,rpo,GRA_FILL_MODE_NONE);
       } else if (type2==5) {
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,5,po,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,5,rpo,GRA_FILL_MODE_EVEN_ODD);
         po[1]=y0-size/4;
         po[2]=x0+size/4;
         po[5]=y0+size/4;
         po[6]=x0-size/4;
         po[9]=y0-size/4;
-        GRAcolor(GC,br,bg,bb, ba);
-        GRAdrawpoly(GC,5,po,GRA_FILL_MODE_EVEN_ODD);
+	GRArotate(x0, y0, po, rpo, 5, dx, dy);
+	GRAcolor(GC,br,bg,bb, ba);
+        GRAdrawpoly(GC,5,rpo,GRA_FILL_MODE_EVEN_ODD);
       } else {
         GRAcolor(GC,br,bg,bb, ba);
-        GRAdrawpoly(GC,5,po,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,5,rpo,GRA_FILL_MODE_EVEN_ODD);
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,5,po,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,5,rpo,GRA_FILL_MODE_NONE);
         if (type2==3) {
           po[1]=y0-size/4;
           po[2]=x0+size/4;
           po[5]=y0+size/4;
           po[6]=x0-size/4;
           po[9]=y0-size/4;
-          GRAdrawpoly(GC,5,po,GRA_FILL_MODE_NONE);
+	  GRArotate(x0, y0, po, rpo, 5, dx, dy);
+          GRAdrawpoly(GC,5,rpo,GRA_FILL_MODE_NONE);
         } else if (type2!=1) {
           if (type2==4) {
             po[1]=y0-size/4;
@@ -1342,25 +1420,30 @@ GRAmark(int GC,int type,int x0,int y0,int size,
             po[5]=y0+size/4;
             po[6]=x0-size/4;
             po[9]=y0-size/4;
-            GRAdrawpoly(GC,5,po,GRA_FILL_MODE_EVEN_ODD);
+	    GRArotate(x0, y0, po, rpo, 5, dx, dy);
+            GRAdrawpoly(GC,5,rpo,GRA_FILL_MODE_EVEN_ODD);
           } else if (type2==6) {
             po[6]=x0;
             po[7]=y0;
-            GRAdrawpoly(GC,5,po,GRA_FILL_MODE_EVEN_ODD);
+	    GRArotate(x0, y0, po, rpo, 5, dx, dy);
+            GRAdrawpoly(GC,5,rpo,GRA_FILL_MODE_EVEN_ODD);
           } else if (type2==7) {
             po[2]=x0;
             po[3]=y0;
-            GRAdrawpoly(GC,5,po,GRA_FILL_MODE_EVEN_ODD);
+	    GRArotate(x0, y0, po, rpo, 5, dx, dy);
+            GRAdrawpoly(GC,5,rpo,GRA_FILL_MODE_EVEN_ODD);
           } else if (type2==8) {
             po[4]=x0;
             po[5]=y0;
+	    GRArotate(x0, y0, rpo, rpo, 5, dx, dy);
             GRAdrawpoly(GC,5,po,GRA_FILL_MODE_EVEN_ODD);
           } else if (type2==9) {
             po[0]=x0;
             po[1]=y0;
             po[8]=x0;
             po[9]=y0;
-            GRAdrawpoly(GC,5,po,GRA_FILL_MODE_EVEN_ODD);
+	    GRArotate(x0, y0, po, rpo, 5, dx, dy);
+            GRAdrawpoly(GC,5,rpo,GRA_FILL_MODE_EVEN_ODD);
           }
         }
       }
@@ -1385,28 +1468,30 @@ GRAmark(int GC,int type,int x0,int y0,int size,
       po[5]=y0+sgn*size/4;
       po[6]=x0;
       po[7]=y0-sgn*size/2;
+      GRArotate(x0, y0, po, rpo, 4, dx, dy);
       if (type2==0) {
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
       } else if (type2==2) {
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_NONE);
       } else if (type2==5) {
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
         po[1]=y0-sgn*size/d/2;
         po[2]=x0+size/4;
         po[3]=y0+sgn*size/d/4;
         po[4]=x0-size/4;
         po[5]=y0+sgn*size/d/4;
         po[7]=y0-sgn*size/d/2;
+	GRArotate(x0, y0, po, rpo, 4, dx, dy);
         GRAcolor(GC,br,bg,bb, ba);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
       } else {
         GRAcolor(GC,br,bg,bb, ba);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_NONE);
         if (type2==3) {
           po[1]=y0-sgn*size/d/2;
           po[2]=x0+size/4;
@@ -1414,7 +1499,8 @@ GRAmark(int GC,int type,int x0,int y0,int size,
           po[4]=x0-size/4;
           po[5]=y0+sgn*size/d/4;
           po[7]=y0-sgn*size/d/2;
-          GRAdrawpoly(GC,4,po,GRA_FILL_MODE_NONE);
+	  GRArotate(x0, y0, po, rpo, 4, dx, dy);
+          GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_NONE);
         } else if (type2!=1) {
           if (type2==4) {
             po[1]=y0-sgn*size/d/2;
@@ -1423,13 +1509,16 @@ GRAmark(int GC,int type,int x0,int y0,int size,
             po[4]=x0-size/4;
             po[5]=y0+sgn*size/d/4;
             po[7]=y0-sgn*size/d/2;
-            GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
+	    GRArotate(x0, y0, po, rpo, 4, dx, dy);
+            GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
           } else if (type2==6) {
             po[4]=x0;
-            GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
+	    GRArotate(x0, y0, po, rpo, 4, dx, dy);
+            GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
           } else if (type2==7) {
             po[2]=x0;
-            GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
+	    GRArotate(x0, y0, po, rpo, 4, dx, dy);
+            GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
           }
         }
       }
@@ -1454,28 +1543,30 @@ GRAmark(int GC,int type,int x0,int y0,int size,
       po[5]=y0-size*d/4;
       po[6]=x0-sgn*size/2;
       po[7]=y0;
+      GRArotate(x0, y0, po, rpo, 4, dx, dy);
       if (type2==0) {
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
       } else if (type2==2) {
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_NONE);
       } else if (type2==5) {
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
         po[0]=x0-sgn*size/d/2;
         po[2]=x0+sgn*size/d/4;
         po[3]=y0+size/4;
         po[4]=x0+sgn*size/d/4;
         po[5]=y0-size/4;
         po[6]=x0-sgn*size/d/2;
+	GRArotate(x0, y0, po, rpo, 4, dx, dy);
         GRAcolor(GC,br,bg,bb, ba);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
       } else {
         GRAcolor(GC,br,bg,bb, ba);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_NONE);
         if (type2==3) {
           po[0]=x0-sgn*size/d/2;
           po[2]=x0+sgn*size/d/4;
@@ -1483,7 +1574,8 @@ GRAmark(int GC,int type,int x0,int y0,int size,
           po[4]=x0+sgn*size/d/4;
           po[5]=y0-size/4;
           po[6]=x0-sgn*size/d/2;
-          GRAdrawpoly(GC,4,po,GRA_FILL_MODE_NONE);
+	  GRArotate(x0, y0, po, rpo, 4, dx, dy);
+          GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_NONE);
         } else if (type2!=1) {
           if (type2==4) {
             po[0]=x0-sgn*size/d/2;
@@ -1492,13 +1584,16 @@ GRAmark(int GC,int type,int x0,int y0,int size,
             po[4]=x0+sgn*size/d/4;
             po[5]=y0-size/4;
             po[6]=x0-sgn*size/d/2;
-            GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
+	    GRArotate(x0, y0, po, rpo, 4, dx, dy);
+            GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
           } else if (type2==6) {
             po[5]=y0;
-            GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
+	    GRArotate(x0, y0, po, rpo, 4, dx, dy);
+            GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
           } else if (type2==7) {
             po[3]=y0;
-            GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
+	    GRArotate(x0, y0, po, rpo, 4, dx, dy);
+            GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
           }
         }
       }
@@ -1520,31 +1615,33 @@ GRAmark(int GC,int type,int x0,int y0,int size,
       po2[5]=y0+size/2;
       po2[6]=x0;
       po2[7]=y0;
+      GRArotate(x0, y0, po, rpo, 4, dx, dy);
+      GRArotate(x0, y0, po2, rpo2, 4, dx, dy);
       if (type==38) {
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
-        GRAdrawpoly(GC,4,po2,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo2,GRA_FILL_MODE_EVEN_ODD);
       } else if (type==39) {
         GRAcolor(GC,br,bg,bb, ba);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
-        GRAdrawpoly(GC,4,po2,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo2,GRA_FILL_MODE_EVEN_ODD);
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_NONE);
-        GRAdrawpoly(GC,4,po2,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,4,rpo2,GRA_FILL_MODE_NONE);
       } else if (type==48) {
         GRAcolor(GC,br,bg,bb, ba);
-        GRAdrawpoly(GC,4,po2,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo2,GRA_FILL_MODE_EVEN_ODD);
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_NONE);
-        GRAdrawpoly(GC,4,po2,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,4,rpo2,GRA_FILL_MODE_NONE);
       } else if (type==49) {
         GRAcolor(GC,br,bg,bb, ba);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,4,po2,GRA_FILL_MODE_EVEN_ODD);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_NONE);
-        GRAdrawpoly(GC,4,po2,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,4,rpo2,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,4,rpo2,GRA_FILL_MODE_NONE);
       }
       break;
     case 58: case 59: case 68: case 69:
@@ -1564,93 +1661,177 @@ GRAmark(int GC,int type,int x0,int y0,int size,
       po2[5]=y0+size/2;
       po2[6]=x0;
       po2[7]=y0;
+      GRArotate(x0, y0, po, rpo, 4, dx, dy);
+      GRArotate(x0, y0, po2, rpo2, 4, dx, dy);
       if (type==58) {
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
-        GRAdrawpoly(GC,4,po2,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo2,GRA_FILL_MODE_EVEN_ODD);
       } else if (type==59) {
         GRAcolor(GC,br,bg,bb, ba);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
-        GRAdrawpoly(GC,4,po2,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo2,GRA_FILL_MODE_EVEN_ODD);
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_NONE);
-        GRAdrawpoly(GC,4,po2,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,4,rpo2,GRA_FILL_MODE_NONE);
       } else if (type==68) {
         GRAcolor(GC,br,bg,bb, ba);
-        GRAdrawpoly(GC,4,po2,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo2,GRA_FILL_MODE_EVEN_ODD);
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_NONE);
-        GRAdrawpoly(GC,4,po2,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,4,rpo2,GRA_FILL_MODE_NONE);
       } else if (type==69) {
         GRAcolor(GC,br,bg,bb, ba);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_EVEN_ODD);
         GRAcolor(GC,fr,fg,fb, fa);
-        GRAdrawpoly(GC,4,po2,GRA_FILL_MODE_EVEN_ODD);
-        GRAdrawpoly(GC,4,po,GRA_FILL_MODE_NONE);
-        GRAdrawpoly(GC,4,po2,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,4,rpo2,GRA_FILL_MODE_EVEN_ODD);
+        GRAdrawpoly(GC,4,rpo,GRA_FILL_MODE_NONE);
+        GRAdrawpoly(GC,4,rpo2,GRA_FILL_MODE_NONE);
       }
       break;
     case 70:
+      po[0] = x0-size/2;
+      po[1] = y0;
+      po[2] = x0+size/2;
+      po[3] = y0;
+      po[4] = x0;
+      po[5] = y0-size/2;
+      po[6] = x0;
+      po[7] = y0+size/2;
+      GRArotate(x0, y0, po, rpo, 4, dx, dy);
       GRAcolor(GC,fr,fg,fb, fa);
-      GRAline(GC,x0-size/2,y0,x0+size/2,y0);
-      GRAline(GC,x0,y0-size/2,x0,y0+size/2);
+      GRAline(GC,rpo[0],rpo[1],rpo[2],rpo[3]);
+      GRAline(GC,rpo[4],rpo[5],rpo[6],rpo[7]);
       break;
     case 71:
       d=sqrt(2.0);
+      po[0] = x0-size/d/2;
+      po[1] = y0-size/d/2;
+      po[2] = x0+size/d/2;
+      po[3] = y0+size/d/2;
+      po[4] = x0+size/d/2;
+      po[5] = y0-size/d/2;
+      po[6] = x0-size/d/2;
+      po[7] = y0+size/d/2;
+      GRArotate(x0, y0, po, rpo, 4, dx, dy);
       GRAcolor(GC,fr,fg,fb, fa);
-      GRAline(GC,x0-size/d/2,y0-size/d/2,x0+size/d/2,y0+size/d/2);
-      GRAline(GC,x0+size/d/2,y0-size/d/2,x0-size/d/2,y0+size/d/2);
+      GRAline(GC,rpo[0],rpo[1],rpo[2],rpo[3]);
+      GRAline(GC,rpo[4],rpo[5],rpo[6],rpo[7]);
       break;
     case 72:
       d=sqrt(3.0);
+      po[0] = x0;
+      po[1] = y0+size/2;
+      po[2] = x0;
+      po[3] = y0-size/2;
+      po[4] = x0+size*d/4;
+      po[5] = y0-size/4;
+      po[6] = x0-size*d/4;
+      po[7] = y0+size/4;
+      po[8] = x0-size*d/4;
+      po[9] = y0-size/4;
+      po[10] = x0+size*d/4;
+      po[11] = y0+size/4;
+      GRArotate(x0, y0, po, rpo, 6, dx, dy);
       GRAcolor(GC,fr,fg,fb, fa);
-      GRAline(GC,x0,y0+size/2,x0,y0-size/2);
-      GRAline(GC,x0+size*d/4,y0-size/4,x0-size*d/4,y0+size/4);
-      GRAline(GC,x0-size*d/4,y0-size/4,x0+size*d/4,y0+size/4);
+      GRAline(GC,rpo[0],rpo[1],rpo[2],rpo[3]);
+      GRAline(GC,rpo[4],rpo[5],rpo[6],rpo[7]);
+      GRAline(GC,rpo[8],rpo[9],rpo[10],rpo[11]);
       break;
     case 73:
       d=sqrt(3.0);
+      po[0] = x0+size/2;
+      po[1] = y0;
+      po[2] = x0-size/2;
+      po[3] = y0;
+      po[4] = x0-size/4;
+      po[5] = y0+size*d/4;
+      po[6] = x0+size/4;
+      po[7] = y0-size*d/4;
+      po[8] = x0-size/4;
+      po[9] = y0-size*d/4;
+      po[10] = x0+size/4;
+      po[11] = y0+size*d/4;
+      GRArotate(x0, y0, po, rpo, 6, dx, dy);
       GRAcolor(GC,fr,fg,fb, fa);
-      GRAline(GC,x0+size/2,y0,x0-size/2,y0);
-      GRAline(GC,x0-size/4,y0+size*d/4,x0+size/4,y0-size*d/4);
-      GRAline(GC,x0-size/4,y0-size*d/4,x0+size/4,y0+size*d/4);
+      GRAline(GC,rpo[0],rpo[1],rpo[2],rpo[3]);
+      GRAline(GC,rpo[4],rpo[5],rpo[6],rpo[7]);
+      GRAline(GC,rpo[8],rpo[9],rpo[10],rpo[11]);
       break;
     case 74:
       d=sqrt(3.0);
+      po[0] = x0;
+      po[1] = y0-size/2;
+      po[2] = x0-size*d/4;
+      po[3] = y0+size/4;
+      po[4] = x0+size*d/4;
+      po[5] = y0+size/4;
+      GRArotate(x0, y0, po, rpo, 3, dx, dy);
       GRAcolor(GC,fr,fg,fb, fa);
-      GRAline(GC,x0,y0,x0,y0-size/2);
-      GRAline(GC,x0,y0,x0-size*d/4,y0+size/4);
-      GRAline(GC,x0,y0,x0+size*d/4,y0+size/4);
+      GRAline(GC,x0,y0,rpo[0],rpo[1]);
+      GRAline(GC,x0,y0,rpo[2],rpo[3]);
+      GRAline(GC,x0,y0,rpo[4],rpo[5]);
       break;
     case 75:
       d=sqrt(3.0);
+      po[0] = x0;
+      po[1] = y0+size/2;
+      po[2] = x0-size*d/4;
+      po[3] = y0-size/4;
+      po[4] = x0+size*d/4;
+      po[5] = y0-size/4;
+      GRArotate(x0, y0, po, rpo, 3, dx, dy);
       GRAcolor(GC,fr,fg,fb, fa);
-      GRAline(GC,x0,y0,x0,y0+size/2);
-      GRAline(GC,x0,y0,x0-size*d/4,y0-size/4);
-      GRAline(GC,x0,y0,x0+size*d/4,y0-size/4);
+      GRAline(GC,x0,y0,rpo[0],rpo[1]);
+      GRAline(GC,x0,y0,rpo[2],rpo[3]);
+      GRAline(GC,x0,y0,rpo[4],rpo[5]);
       break;
     case 76:
       d=sqrt(3.0);
+      po[0] = x0-size/2;
+      po[1] = y0;
+      po[2] = x0+size/4;
+      po[3] = y0-size*d/4;
+      po[4] = x0+size/4;
+      po[5] = y0+size*d/4;
+      GRArotate(x0, y0, po, rpo, 3, dx, dy);
       GRAcolor(GC,fr,fg,fb, fa);
-      GRAline(GC,x0,y0,x0-size/2,y0);
-      GRAline(GC,x0,y0,x0+size/4,y0-size*d/4);
-      GRAline(GC,x0,y0,x0+size/4,y0+size*d/4);
+      GRAline(GC,x0,y0,rpo[0],rpo[1]);
+      GRAline(GC,x0,y0,rpo[2],rpo[3]);
+      GRAline(GC,x0,y0,rpo[4],rpo[5]);
       break;
     case 77:
       d=sqrt(3.0);
+      po[0] = x0+size/2;
+      po[1] = y0;
+      po[2] = x0-size/4;
+      po[3] = y0-size*d/4;
+      po[4] = x0-size/4;
+      po[5] = y0+size*d/4;
+      GRArotate(x0, y0, po, rpo, 3, dx, dy);
       GRAcolor(GC,fr,fg,fb, fa);
-      GRAline(GC,x0,y0,x0+size/2,y0);
-      GRAline(GC,x0,y0,x0-size/4,y0-size*d/4);
-      GRAline(GC,x0,y0,x0-size/4,y0+size*d/4);
+      GRAline(GC,x0,y0,rpo[0],rpo[1]);
+      GRAline(GC,x0,y0,rpo[2],rpo[3]);
+      GRAline(GC,x0,y0,rpo[4],rpo[5]);
       break;
     case 78:
       GRAcolor(GC,fr,fg,fb, fa);
+      po[0] = x0-size/2;
+      po[1] = y0;
+      po[2] = x0+size/2;
+      po[3] = y0;
+      GRArotate(x0, y0, po, rpo, 4, dx, dy);
       GRAline(GC,x0-size/2,y0,x0+size/2,y0);
       break;
     case 79:
       GRAcolor(GC,fr,fg,fb, fa);
-      GRAline(GC,x0,y0-size/2,x0,y0+size/2);
+      po[0] = x0;
+      po[1] = y0-size/2;
+      po[2] = x0;
+      po[3] = y0+size/2;
+      GRArotate(x0, y0, po, rpo, 4, dx, dy);
+      GRAline(GC, rpo[0], rpo[1], rpo[2], rpo[3]);
       break;
     case 80:
       r=size/2;
@@ -1663,16 +1844,34 @@ GRAmark(int GC,int type,int x0,int y0,int size,
       r=size/2;
       GRAcolor(GC,fr,fg,fb, fa);
       GRAcircle(GC,x0,y0,r,r,0,36000,0);
-      GRAline(GC,x0-size/2,y0,x0+size/2,y0);
-      GRAline(GC,x0,y0-size/2,x0,y0+size/2);
+      po[0] = x0-size/2;
+      po[1] = y0;
+      po[2] = x0+size/2;
+      po[3] = y0;
+      po[4] = x0;
+      po[5] = y0-size/2;
+      po[6] = x0;
+      po[7] = y0+size/2;
+      GRArotate(x0, y0, po, rpo, 4, dx, dy);
+      GRAline(GC, rpo[0], rpo[1], rpo[2], rpo[3]);
+      GRAline(GC, rpo[4], rpo[5], rpo[6], rpo[7]);
       break;
     case 82:
       r=size/2;
       d=sqrt(2.0);
       GRAcolor(GC,fr,fg,fb, fa);
       GRAcircle(GC,x0,y0,r,r,0,36000,0);
-      GRAline(GC,x0-size/d/2,y0-size/d/2,x0+size/d/2,y0+size/d/2);
-      GRAline(GC,x0-size/d/2,y0+size/d/2,x0+size/d/2,y0-size/d/2);
+      po[0] = x0-size/d/2;
+      po[1] = y0-size/d/2;
+      po[2] = x0+size/d/2;
+      po[3] = y0+size/d/2;
+      po[4] = x0-size/d/2;
+      po[5] = y0+size/d/2;
+      po[6] = x0+size/d/2;
+      po[7] = y0-size/d/2;
+      GRArotate(x0, y0, po, rpo, 4, dx, dy);
+      GRAline(GC, rpo[0], rpo[1], rpo[2], rpo[3]);
+      GRAline(GC, rpo[4], rpo[5], rpo[6], rpo[7]);
       break;
     case 83:
       x1=x0-size/2;
@@ -1680,12 +1879,12 @@ GRAmark(int GC,int type,int x0,int y0,int size,
       x2=x0+size/2;
       y2=y0+size/2;
       GRAcolor(GC,fr,fg,fb, fa);
-      GRArectangle(GC,x1,y1,x2,y2,0);
+      GRArectangle_rotate(GC,dx,dy,x0,y0,x1,y1,x2,y2,0);
       x1=x0-size/4;
       y1=y0-size/4;
       x2=x0+size/4;
       y2=y0+size/4;
-      GRArectangle(GC,x1,y1,x2,y2,0);
+      GRArectangle_rotate(GC,dx,dy,x0,y0,x1,y1,x2,y2,0);
       break;
     case 84:
       x1=x0-size/2;
@@ -1693,9 +1892,18 @@ GRAmark(int GC,int type,int x0,int y0,int size,
       x2=x0+size/2;
       y2=y0+size/2;
       GRAcolor(GC,fr,fg,fb, fa);
-      GRArectangle(GC,x1,y1,x2,y2,0);
-      GRAline(GC,x0-size/2,y0,x0+size/2,y0);
-      GRAline(GC,x0,y0-size/2,x0,y0+size/2);
+      GRArectangle_rotate(GC,dx,dy,x0,y0,x1,y1,x2,y2,0);
+      po[0] = x0-size/2;
+      po[1] = y0;
+      po[2] = x0+size/2;
+      po[3] = y0;
+      po[4] = x0;
+      po[5] = y0-size/2;
+      po[6] = x0;
+      po[7] = y0+size/2;
+      GRArotate(x0, y0, po, rpo, 4, dx, dy);
+      GRAline(GC, rpo[0], rpo[1], rpo[2], rpo[3]);
+      GRAline(GC, rpo[4], rpo[5], rpo[6], rpo[7]);
       break;
     case 85:
       x1=x0-size/2;
@@ -1703,9 +1911,18 @@ GRAmark(int GC,int type,int x0,int y0,int size,
       x2=x0+size/2;
       y2=y0+size/2;
       GRAcolor(GC,fr,fg,fb, fa);
-      GRArectangle(GC,x1,y1,x2,y2,0);
-      GRAline(GC,x0-size/2,y0-size/2,x0+size/2,y0+size/2);
-      GRAline(GC,x0+size/2,y0-size/2,x0-size/2,y0+size/2);
+      GRArectangle_rotate(GC,dx,dy,x0,y0,x1,y1,x2,y2,0);
+      po[0] = x0-size/2;
+      po[1] = y0-size/2;
+      po[2] = x0+size/2;
+      po[3] = y0+size/2;
+      po[4] = x0+size/2;
+      po[5] = y0-size/2;
+      po[6] = x0-size/2;
+      po[7] = y0+size/2;
+      GRArotate(x0, y0, po, rpo, 4, dx, dy);
+      GRAline(GC, rpo[0], rpo[1], rpo[2], rpo[3]);
+      GRAline(GC, rpo[4], rpo[5], rpo[6], rpo[7]);
       break;
     case 86:
       po[0]=x0;
@@ -1718,14 +1935,16 @@ GRAmark(int GC,int type,int x0,int y0,int size,
       po[7]=y0;
       po[8]=x0;
       po[9]=y0-size/2;
+      GRArotate(x0, y0, po, rpo, 5, dx, dy);
       GRAcolor(GC,fr,fg,fb, fa);
-      GRAdrawpoly(GC,5,po,GRA_FILL_MODE_NONE);
+      GRAdrawpoly(GC,5,rpo,GRA_FILL_MODE_NONE);
       po[1]=y0-size/4;
       po[2]=x0+size/4;
       po[5]=y0+size/4;
       po[6]=x0-size/4;
       po[9]=y0-size/4;
-      GRAdrawpoly(GC,5,po,GRA_FILL_MODE_NONE);
+      GRArotate(x0, y0, po, rpo, 5, dx, dy);
+      GRAdrawpoly(GC,5,rpo,GRA_FILL_MODE_NONE);
       break;
     case 87:
       po[0]=x0;
@@ -1738,10 +1957,20 @@ GRAmark(int GC,int type,int x0,int y0,int size,
       po[7]=y0;
       po[8]=x0;
       po[9]=y0-size/2;
+      GRArotate(x0, y0, po, rpo, 5, dx, dy);
       GRAcolor(GC,fr,fg,fb, fa);
-      GRAdrawpoly(GC,5,po,GRA_FILL_MODE_NONE);
-      GRAline(GC,x0-size/2,y0,x0+size/2,y0);
-      GRAline(GC,x0,y0-size/2,x0,y0+size/2);
+      GRAdrawpoly(GC,5,rpo,GRA_FILL_MODE_NONE);
+      po[0] = x0-size/2;
+      po[1] = y0;
+      po[2] = x0+size/2;
+      po[3] = y0;
+      po[4] = x0;
+      po[5] = y0-size/2;
+      po[6] = x0;
+      po[7] = y0+size/2;
+      GRArotate(x0, y0, po, rpo, 4, dx, dy);
+      GRAline(GC, rpo[0], rpo[1], rpo[2], rpo[3]);
+      GRAline(GC, rpo[4], rpo[5], rpo[6], rpo[7]);
       break;
     case 88:
       po[0]=x0;
@@ -1754,10 +1983,20 @@ GRAmark(int GC,int type,int x0,int y0,int size,
       po[7]=y0;
       po[8]=x0;
       po[9]=y0-size/2;
+      GRArotate(x0, y0, po, rpo, 5, dx, dy);
       GRAcolor(GC,fr,fg,fb, fa);
-      GRAdrawpoly(GC,5,po,GRA_FILL_MODE_NONE);
-      GRAline(GC,x0-size/4,y0-size/4,x0+size/4,y0+size/4);
-      GRAline(GC,x0-size/4,y0+size/4,x0+size/4,y0-size/4);
+      GRAdrawpoly(GC,5,rpo,GRA_FILL_MODE_NONE);
+      po[0] = x0-size/4;
+      po[1] = y0-size/4;
+      po[2] = x0+size/4;
+      po[3] = y0+size/4;
+      po[4] = x0-size/4;
+      po[5] = y0+size/4;
+      po[6] = x0+size/4;
+      po[7] = y0-size/4;
+      GRArotate(x0, y0, po, rpo, 4, dx, dy);
+      GRAline(GC, rpo[0], rpo[1], rpo[2], rpo[3]);
+      GRAline(GC, rpo[4], rpo[5], rpo[6], rpo[7]);
       break;
     default:
       GRAcolor(GC,fr,fg,fb, fa);
@@ -1767,6 +2006,13 @@ GRAmark(int GC,int type,int x0,int y0,int size,
   }
 }
 
+void
+GRAmark(int GC,int type,int x0,int y0,int size,
+	int fr,int fg,int fb, int fa, int br,int bg,int bb, int ba)
+{
+  GRAmark_rotate(GC, type, x0, y0, 1, 0, size, fr, fg, fb, fa, br, bg, bb, ba);
+}
+
 void
 GRAouttext(int GC,char *s)
 {
@@ -2803,12 +3049,25 @@ void
 GRAtextextentraw(char *s,char *font, int style,
 		 int size,int space,int *gx0,int *gy0,int *gx1,int *gy1)
 {
+  int i, n, len, ha, hd;
   *gx0 = *gy0 = *gx1 = *gy1 = 0;
   if (s == NULL || font == NULL) return;
 
+  len = strlen(s);
+  if (len < 1) {
+    return;
+  }
   *gx1 = GRAstrwidth(s, font, style, size)
-    + nround(space / 72.0 * 25.4) * (strlen(s) - 1);
-  *gy0 = - GRAcharascent(font, style, size);
+    + nround(space / 72.0 * 25.4) * (len - 1);
+  for (n = 0, i = 0; i < len; i++) {
+    if (s[i] == '\n') {
+      n++;
+    }
+  }
+  ha = GRAcharascent(font, style, size);
+  *gy0 = - ha;
+  hd = GRAchardescent(font, style, size);
+  *gy1 = hd + (ha + hd) * n;
 }
 
 
@@ -2843,14 +3102,19 @@ getintpar(char *s,int num,int cpar[])
 }
 
 int
-GRAinputdraw(int GC,int leftm,int topm,int rate,
+GRAinputdraw(int GC,int leftm,int topm,int rate_x,int rate_y,
                   char code,int *cpar,char *cstr)
 {
   int i;
-  double r;
+  double r, rx, ry;
 
-  if (GRAClist[GC].mergezoom==0) r=1;
-  else r=((double )rate)/GRAClist[GC].mergezoom;
+  if (GRAClist[GC].mergezoom==0) {
+    rx = ry = 1;
+  } else {
+    rx = ((double )rate_x)/GRAClist[GC].mergezoom;
+    ry = ((double )rate_y)/GRAClist[GC].mergezoom;
+  }
+  r = MIN(rx, ry);
   switch (code) {
   case '%':
   case 'O': case 'Q': case 'F': case 'S': case 'K': case 'Z':
@@ -2878,10 +3142,10 @@ GRAinputdraw(int GC,int leftm,int topm,int rate,
     if (cpar[0] != 5)
       return FALSE;
 
-    cpar[1]=(int )(((cpar[1]-GRAClist[GC].mergeleft)*r)+leftm);
-    cpar[2]=(int )(((cpar[2]-GRAClist[GC].mergetop)*r)+topm);
-    cpar[3]=(int )(((cpar[3]-GRAClist[GC].mergeleft)*r)+leftm);
-    cpar[4]=(int )(((cpar[4]-GRAClist[GC].mergetop)*r)+topm);
+    cpar[1]=(int )(((cpar[1]-GRAClist[GC].mergeleft)*rx)+leftm);
+    cpar[2]=(int )(((cpar[2]-GRAClist[GC].mergetop)*ry)+topm);
+    cpar[3]=(int )(((cpar[3]-GRAClist[GC].mergeleft)*rx)+leftm);
+    cpar[4]=(int )(((cpar[4]-GRAClist[GC].mergetop)*ry)+topm);
     break;
   case 'A':
     if (cpar[0] != cpar[1] + 5)
@@ -2891,57 +3155,60 @@ GRAinputdraw(int GC,int leftm,int topm,int rate,
     for (i=0;i<cpar[1];i++) cpar[i+5]=(int )(cpar[i+5]*r);
     break;
   case 'H':
-    if (cpar[0] != 3 && cpar[0] != 4)
+    if (cpar[0] != 3 && cpar[0] != 4) {
       return FALSE;
-
-    cpar[1]=(int )(cpar[1]*r);
-    cpar[2]=(int )(cpar[2]*r);
+    } else {
+      double rp, rn;
+      cpar[3] = calc_zoom_direction(cpar[3], rx, ry, &rp, &rn);
+      cpar[1]=(int )(cpar[1]*MIN(rp, rn));
+      cpar[2]=(int )(cpar[2]*rp);
+    }
     break;
   case 'M': case 'N': case 'T': case 'P':
     if (cpar[0] != 2)
       return FALSE;
 
-    cpar[1]=(int )(cpar[1]*r);
-    cpar[2]=(int )(cpar[2]*r);
+    cpar[1]=(int )(cpar[1]*rx);
+    cpar[2]=(int )(cpar[2]*ry);
     break;
   case 'L':
     if (cpar[0] != 4)
       return FALSE;
 
-    cpar[1]=(int )(cpar[1]*r);
-    cpar[2]=(int )(cpar[2]*r);
-    cpar[3]=(int )(cpar[3]*r);
-    cpar[4]=(int )(cpar[4]*r);
+    cpar[1]=(int )(cpar[1]*rx);
+    cpar[2]=(int )(cpar[2]*ry);
+    cpar[3]=(int )(cpar[3]*rx);
+    cpar[4]=(int )(cpar[4]*ry);
     break;
   case 'C':
     if (cpar[0] != 7)
       return FALSE;
 
-    cpar[1]=(int )(cpar[1]*r);
-    cpar[2]=(int )(cpar[2]*r);
-    cpar[3]=(int )(cpar[3]*r);
-    cpar[4]=(int )(cpar[4]*r);
+    cpar[1]=(int )(cpar[1]*rx);
+    cpar[2]=(int )(cpar[2]*ry);
+    cpar[3]=(int )(cpar[3]*rx);
+    cpar[4]=(int )(cpar[4]*ry);
     break;
   case 'B':
     if (cpar[0] != 5)
       return FALSE;
 
-    cpar[1]=(int )(cpar[1]*r);
-    cpar[2]=(int )(cpar[2]*r);
-    cpar[3]=(int )(cpar[3]*r);
-    cpar[4]=(int )(cpar[4]*r);
+    cpar[1]=(int )(cpar[1]*rx);
+    cpar[2]=(int )(cpar[2]*ry);
+    cpar[3]=(int )(cpar[3]*rx);
+    cpar[4]=(int )(cpar[4]*ry);
     break;
   case 'R':
     if (cpar[0] != cpar[1] * 2 + 1)
       return FALSE;
 
-    for (i=0;i<(2*(cpar[1]));i++) cpar[i+2]=(int )(cpar[i+2]*r);
+    for (i=0;i<(2*(cpar[1]));i++) cpar[i+2]=(int )(cpar[i+2]*((i % 2) ? ry : rx));
     break;
   case 'D':
     if (cpar[0] != cpar[1] * 2 + 2)
       return FALSE;
 
-    for (i=0;i<(2*(cpar[1]));i++) cpar[i+3]=(int )(cpar[i+3]*r);
+    for (i=0;i<(2*(cpar[1]));i++) cpar[i+3]=(int )(cpar[i+3]*((i % 2) ? ry : rx));
     break;
   }
   if (code!='\0') GRAdraw(GC,code,cpar,cstr);
@@ -3033,7 +3300,7 @@ errexit:
 }
 
 int
-GRAinput(int GC,char *s,int leftm,int topm,int rate)
+GRAinput(int GC,char *s,int leftm,int topm,int rate_x,int rate_y)
 {
   int r;
   struct GRAdata data;
@@ -3041,7 +3308,7 @@ GRAinput(int GC,char *s,int leftm,int topm,int rate)
   if (! GRAparse(&data, s)) {
     return FALSE;
   }
-  r = GRAinputdraw(GC,leftm,topm,rate,data.code,data.cpar,data.cstr);
+  r = GRAinputdraw(GC,leftm,topm,rate_x,rate_y,data.code,data.cpar,data.cstr);
   GRAdata_free(&data);
   return r;
 }
@@ -3068,7 +3335,7 @@ get_gra_font(int i)
 }
 
 int
-GRAinputold(int GC,char *s,int leftm,int topm,int rate)
+GRAinputold(int GC,char *s,int leftm,int topm,int rate_x,int rate_y)
 {
   int pos,num,i,j;
   char code,code2;
@@ -3116,12 +3383,12 @@ GRAinputold(int GC,char *s,int leftm,int topm,int rate)
   case 'X':
     break;
   case '%': case 'S': case 'Z':
-    GRAinputdraw(GC,leftm,topm,rate,code,cpar,cstr);
+    GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code,cpar,cstr);
     break;
   case 'K':
     ustr = sjis_to_utf8(cstr);
     if (ustr) {
-      GRAinputdraw(GC, leftm, topm, rate, 'S', cpar, ustr);
+      GRAinputdraw(GC, leftm, topm, rate_x, rate_y, 'S', cpar, ustr);
       g_free(ustr);
     }
     break;
@@ -3130,17 +3397,17 @@ GRAinputold(int GC,char *s,int leftm,int topm,int rate)
     cpar[5]=nround(cpar[3]/2.1);
     cpar[3]=21000;
     cpar[4]=29700;
-    GRAinputdraw(GC,leftm,topm,rate,code,cpar,cstr);
+    GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code,cpar,cstr);
     break;
   case 'E': case 'M': case 'N': case 'L': case 'T': case 'P':
-    GRAinputdraw(GC,leftm,topm,rate,code,cpar,cstr);
+    GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code,cpar,cstr);
     break;
   case 'V':
     if (cpar[0]==4) {
       cpar[0]=5;
       cpar[5]=0;
     }
-    GRAinputdraw(GC,leftm,topm,rate,code,cpar,cstr);
+    GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code,cpar,cstr);
     break;
   case 'A':
     col=cpar[3];
@@ -3152,12 +3419,12 @@ GRAinputold(int GC,char *s,int leftm,int topm,int rate)
     cpar2[1]=GRAClist[GC].oldFR;
     cpar2[2]=GRAClist[GC].oldFG;
     cpar2[3]=GRAClist[GC].oldFB;
-    GRAinputdraw(GC,leftm,topm,rate,code2,cpar2,cstr);
+    GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code2,cpar2,cstr);
     cpar[0]=5;
     cpar[3]=cpar[4];
     cpar[4]=0;
     cpar[5]=1000;
-    GRAinputdraw(GC,leftm,topm,rate,code,cpar,cstr);
+    GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code,cpar,cstr);
     break;
   case 'O':
     col=cpar[1];
@@ -3185,16 +3452,16 @@ GRAinputold(int GC,char *s,int leftm,int topm,int rate)
       cpar2[1]=GRAClist[GC].oldBR;
       cpar2[2]=GRAClist[GC].oldBG;
       cpar2[3]=GRAClist[GC].oldBB;
-      GRAinputdraw(GC,leftm,topm,rate,code2,cpar2,cstr);
+      GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code2,cpar2,cstr);
 	}
-    GRAinputdraw(GC,leftm,topm,rate,code,cpar,cstr);
+    GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code,cpar,cstr);
     if (cpar[7]==1) {
       code2='G';
       cpar2[0]=3;
       cpar2[1]=GRAClist[GC].oldFR;
       cpar2[2]=GRAClist[GC].oldFG;
       cpar2[3]=GRAClist[GC].oldFB;
-      GRAinputdraw(GC,leftm,topm,rate,code2,cpar2,cstr);
+      GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code2,cpar2,cstr);
     }
     break;
   case 'B':
@@ -3204,16 +3471,16 @@ GRAinputold(int GC,char *s,int leftm,int topm,int rate)
       cpar2[1]=GRAClist[GC].oldBR;
       cpar2[2]=GRAClist[GC].oldBG;
       cpar2[3]=GRAClist[GC].oldBB;
-      GRAinputdraw(GC,leftm,topm,rate,code2,cpar2,cstr);
+      GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code2,cpar2,cstr);
     }
-    GRAinputdraw(GC,leftm,topm,rate,code,cpar,cstr);
+    GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code,cpar,cstr);
     if (cpar[5]==1) {
       code2='G';
       cpar2[0]=3;
       cpar2[1]=GRAClist[GC].oldFR;
       cpar2[2]=GRAClist[GC].oldFG;
       cpar2[3]=GRAClist[GC].oldFB;
-      GRAinputdraw(GC,leftm,topm,rate,code2,cpar2,cstr);
+      GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code2,cpar2,cstr);
     }
     break;
   case 'D':
@@ -3223,23 +3490,23 @@ GRAinputold(int GC,char *s,int leftm,int topm,int rate)
       cpar2[1]=GRAClist[GC].oldBR;
       cpar2[2]=GRAClist[GC].oldBG;
       cpar2[3]=GRAClist[GC].oldBB;
-      GRAinputdraw(GC,leftm,topm,rate,code2,cpar2,cstr);
+      GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code2,cpar2,cstr);
     }
-    GRAinputdraw(GC,leftm,topm,rate,code,cpar,cstr);
+    GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code,cpar,cstr);
     if (cpar[2]==1) {
       code2='G';
       cpar2[0]=3;
       cpar2[1]=GRAClist[GC].oldFR;
       cpar2[2]=GRAClist[GC].oldFG;
       cpar2[3]=GRAClist[GC].oldFB;
-      GRAinputdraw(GC,leftm,topm,rate,code2,cpar2,cstr);
+      GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code2,cpar2,cstr);
     }
     break;
   case 'F':
     code2='F';
     cpar2[0]=-1;
     GRAClist[GC].mergefont=cpar[1]*4+cpar[2];
-    GRAinputdraw(GC,leftm,topm,rate,code2,cpar2,get_gra_font(cpar[1]*4+cpar[2]));
+    GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code2,cpar2,get_gra_font(cpar[1]*4+cpar[2]));
     if (cpar[6] == 1) {
       cpar[6]=9000;
     } else if (cpar[6]<0) {
@@ -3253,7 +3520,7 @@ GRAinputold(int GC,char *s,int leftm,int topm,int rate)
     GRAClist[GC].mergept=cpar2[1];
     GRAClist[GC].mergesp=cpar2[2];
     GRAClist[GC].mergedir=cpar2[3];
-    GRAinputdraw(GC,leftm,topm,rate,code2,cpar2,cstr);
+    GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code2,cpar2,cstr);
     col=cpar[5];
     B=(col & 1)*256;
     G=(col & 2)*128;
@@ -3263,7 +3530,7 @@ GRAinputold(int GC,char *s,int leftm,int topm,int rate)
     cpar2[1]=R;
     cpar2[2]=G;
     cpar2[3]=B;
-    GRAinputdraw(GC,leftm,topm,rate,code2,cpar2,cstr);
+    GRAinputdraw(GC,leftm,topm,rate_x,rate_y,code2,cpar2,cstr);
     break;
   }
   return TRUE;
