@@ -143,7 +143,13 @@ struct completion_info completion_info_const[] = {
 {"avx", "AVX", NULL, N_("<b>AVX</b>\nThe average of X"), NULL},
 {"avy", "AVY", NULL, N_("<b>AVY</b>\nThe average of X"), NULL},
 {"axisx", "AXISX", NULL, N_("<b>AXISX</b>\nThe ID of the axis instance specified as X axis"), NULL},
+{"axisx_len", "AXISX_LEN", NULL, N_("<b>AXISX_LEN</b>\nLength of the X axis"), NULL},
+{"axisx_max", "AXISX_MAX", NULL, N_("<b>AXISX_MAX</b>\nThe maximum value of the X axis"), NULL},
+{"axisx_min", "AXISX_MIN", NULL, N_("<b>AXISX_MIN</b>\nThe minimum value of the X axis"), NULL},
 {"axisy", "AXISY", NULL, N_("<b>AXISY</b>\nThe ID of the axis instance specified as Y axis"), NULL},
+{"axisy_len", "AXISY_LEN", NULL, N_("<b>AXISY_LEN</b>\nLength of the Y axis"), NULL},
+{"axisy_max", "AXISY_MAX", NULL, N_("<b>AXISY_MAX</b>\nThe maximum value of the Y axis"), NULL},
+{"axisy_min", "AXISY_MIN", NULL, N_("<b>AXISY_MIN</b>\nThe minimum value of the Y axis"), NULL},
 {"break", "BREAK", NULL, N_("<b>BREAK</b>\nMissing data (data lines are not combined)"), NULL},
 {"colx", "COLX", NULL, N_("<b>COLX</b>\nThe column number specified for X"), NULL},
 {"coly", "COLY", NULL, N_("<b>COLY</b>\nThe column number specified for Y"), NULL},
