@@ -77,7 +77,7 @@ file_select(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointe
     getobj(d->obj, "ext", sel, 0, NULL, &ext);
   }
 
-  parent = (Menulocal.single_window_mode) ? TopLevel : d->parent->Win;
+  parent = TopLevel;
 
   if (nGetOpenFileName(parent, _("Open"), ext, NULL, gtk_entry_get_text(w),
 		       &file, TRUE, Menulocal.changedirectory) == IDOK && file) {
@@ -203,7 +203,7 @@ start_editing(GtkCellRenderer *renderer, GtkCellEditable *editable, gchar *path,
 	char *valstr;
 
 	if (strcmp(list->name, "file") == 0) {
-	  gtk_entry_set_icon_from_icon_name(GTK_ENTRY(editable), GTK_ENTRY_ICON_SECONDARY, "document-open");
+	  gtk_entry_set_icon_from_icon_name(GTK_ENTRY(editable), GTK_ENTRY_ICON_SECONDARY, "document-open-symbolic");
 	  g_signal_connect(editable, "icon-release", G_CALLBACK(file_select), d);
 	}
 	sgetobjfield(d->obj, sel, list->name, NULL, &valstr, FALSE, FALSE, FALSE);
@@ -494,310 +494,6 @@ set_obj_cell_renderer_cb(struct obj_list_data *d, int i, n_list_store *list, GCa
   g_signal_connect(rend, "editing-canceled", G_CALLBACK(cancel_editing), d);
 }
 
-static void
-get_geometry(struct SubWin *d, int *x, int *y, int *w, int *h)
-{
-  switch (d->type) {
-  case TypeFileWin:
-    *w = Menulocal.filewidth;
-    *h = Menulocal.fileheight;
-    *x = Menulocal.filex;
-    *y = Menulocal.filey;
-    break;
-  case TypeAxisWin:
-    *w = Menulocal.axiswidth;
-    *h = Menulocal.axisheight;
-    *x = Menulocal.axisx;
-    *y = Menulocal.axisy;
-    break;
-  case TypeLegendWin:
-    *w = Menulocal.legendwidth;
-    *h = Menulocal.legendheight;
-    *x = Menulocal.legendx;
-    *y = Menulocal.legendy;
-    break;
-  case TypeMergeWin:
-    *w = Menulocal.mergewidth;
-    *h = Menulocal.mergeheight;
-    *x = Menulocal.mergex;
-    *y = Menulocal.mergey;
-    break;
-  case TypeInfoWin:
-    *w = Menulocal.dialogwidth;
-    *h = Menulocal.dialogheight;
-    *x = Menulocal.dialogx;
-    *y = Menulocal.dialogy;
-    break;
-  case TypeCoordWin:
-    *w = Menulocal.coordwidth;
-    *h = Menulocal.coordheight;
-    *x = Menulocal.coordx;
-    *y = Menulocal.coordy;
-    break;
-  default:
-    *w = DEFAULT_GEOMETRY;
-    *h = DEFAULT_GEOMETRY;
-    *x = DEFAULT_GEOMETRY;
-    *y = DEFAULT_GEOMETRY;
-    break;
-  }
-}
-
-static void
-set_geometry(struct SubWin *d, int x, int y, int w, int h)
-{
-  switch (d->type) {
-  case TypeFileWin:
-    Menulocal.filewidth = w;
-    Menulocal.fileheight = h;
-    Menulocal.filex = x;
-    Menulocal.filey = y;
-    break;
-  case TypeAxisWin:
-    Menulocal.axiswidth = w;
-    Menulocal.axisheight = h;
-    Menulocal.axisx = x;
-    Menulocal.axisy = y;
-    break;
-  case TypeLegendWin:
-    Menulocal.legendwidth = w;
-    Menulocal.legendheight = h;
-    Menulocal.legendx = x;
-    Menulocal.legendy = y;
-    break;
-  case TypeMergeWin:
-    Menulocal.mergewidth = w;
-    Menulocal.mergeheight = h;
-    Menulocal.mergex = x;
-    Menulocal.mergey = y;
-    break;
-  case TypeInfoWin:
-    Menulocal.dialogwidth = w;
-    Menulocal.dialogheight = h;
-    Menulocal.dialogx = x;
-    Menulocal.dialogy = y;
-    break;
-  case TypeCoordWin:
-    Menulocal.coordwidth = w;
-    Menulocal.coordheight = h;
-    Menulocal.coordx = x;
-    Menulocal.coordy = y;
-    break;
-  }
-}
-
-static void
-set_visibility(struct SubWin *d, int s)
-{
-  switch (d->type) {
-  case TypeFileWin:
-    Menulocal.fileopen = s;
-    break;
-  case TypeAxisWin:
-    Menulocal.axisopen = s;
-    break;
-  case TypeLegendWin:
-    Menulocal.legendopen = s;
-    break;
-  case TypeMergeWin:
-    Menulocal.mergeopen = s;
-    break;
-  case TypeInfoWin:
-    Menulocal.dialogopen = s;
-    break;
-  case TypeCoordWin:
-    Menulocal.coordopen = s;
-    break;
-  }
-}
-
-#define DEFAULT_WIDTH 240
-#define DEFAULT_HEIGHT 320
-
-void
-sub_window_set_geometry(struct SubWin *d, int resize)
-{
-  int w, h, x, y, x0, y0;
-
-  if (d->Win == NULL) return;
-
-  get_geometry(d, &x, &y, &w, &h);
-
-  if (w == DEFAULT_GEOMETRY) {
-    w = DEFAULT_WIDTH;
-  }
-
-  if (h == DEFAULT_GEOMETRY) {
-    h = DEFAULT_HEIGHT;
-  }
-
-  if ((x != DEFAULT_GEOMETRY) && (y != DEFAULT_GEOMETRY)) {
-    gtk_window_get_position(GTK_WINDOW(TopLevel), &x0, &y0);
-    x += x0;
-    y += y0;
-
-    if (x < 0) {
-      x = 0;
-    }
-    if (y < 0) {
-      y = 0;
-    }
-    gtk_window_move(GTK_WINDOW(d->Win), x, y);
-  }
-
-  gtk_window_set_default_size(GTK_WINDOW(d->Win), w, h);
-
-  if (resize) {
-    gtk_window_resize(GTK_WINDOW(d->Win), w, h);
-  }
-}
-
-static int
-get_window_visibility(struct SubWin *d)
-{
-  GdkWindow *win;
-  GdkWindowState state;
-
-  if (d->Win == NULL) {
-    return 0;
-  }
-
-  win = gtk_widget_get_window(d->Win);
-  if (win == NULL) {
-    return 0;
-  }
-
-  state = gdk_window_get_state(win);
-
-  return ! (state & GDK_WINDOW_STATE_WITHDRAWN);
-}
-
-void
-sub_window_save_geometry(struct SubWin *d)
-{
-  gint x, y, x0, y0, w, h;
-
-  if (Menulocal.single_window_mode || ! get_window_visibility(d)) {
-    return;
-  }
-
-  gtk_window_get_position(GTK_WINDOW(TopLevel), &x0, &y0);
-  get_window_geometry(d->Win, &x, &y, &w, &h);
-  set_geometry(d, x - x0, y - y0, w, h);
-}
-
-void
-sub_window_save_visibility(struct SubWin *d)
-{
-  int state;
-
-  state = get_window_visibility(d);
-  set_visibility(d, state);
-}
-
-static void
-sub_window_hide(struct SubWin *d)
-{
-  if (d->Win) {
-    sub_window_save_geometry(d);
-    gtk_widget_hide(d->Win);
-    d->visible = FALSE;
-    set_toggle_action_widget_state(d->action_widget_id, FALSE);
-  }
-}
-
-static void
-sub_window_show(struct SubWin *d)
-{
-  if (d->Win) {
-    gtk_window_present(GTK_WINDOW(d->Win));
-    d->visible = TRUE;
-    set_toggle_action_widget_state(d->action_widget_id, TRUE);
-  }
-}
-
-static void
-sub_window_show_all(struct SubWin *d)
-{
-  if (d->Win) {
-    gtk_widget_show_all(d->Win);
-  }
-}
-
-void
-sub_window_set_visibility(struct SubWin *d, int state)
-{
-  if (d->Win == NULL) {
-    return;
-  }
-
-  if (state && ! Menulocal.single_window_mode) {
-    if (! gtk_widget_get_realized(d->Win)) {
-      sub_window_show_all(d);
-      sub_window_set_geometry(d, TRUE);
-    }
-    sub_window_show(d);
-  } else {
-    sub_window_hide(d);
-  }
-}
-
-static void
-cb_show(GtkWidget *widget, gpointer user_data)
-{
-  struct SubWin *d;
-
-  d = user_data;
-
-  if (d->Win == NULL) return;
-
-  sub_window_set_geometry(d, FALSE);
-}
-
-static gboolean
-cb_del(GtkWidget *w, GdkEvent *event, gpointer user_data)
-{
-  struct SubWin *d;
-
-  d = (struct SubWin *) user_data;
-  set_subwindow_state(d->type, SUBWIN_STATE_HIDE);
-
-  return TRUE;
-}
-
-static void
-cb_destroy(GtkWidget *w, gpointer user_data)
-{
-  struct SubWin *d;
-  struct obj_list_data *ptr, *next;
-
-  d = user_data;
-
-  d->Win = NULL;
-
-  switch (d->type) {
-  case TypeFileWin:
-  case TypeAxisWin:
-  case TypeMergeWin:
-  case TypeLegendWin:
-    for (ptr = d->data.data; ptr; ptr = next) {
-      if (ptr->popup) {
-	gtk_widget_destroy(ptr->popup);
-      }
-      if (ptr->popup_item) {
-	g_free(ptr->popup_item);
-      }
-      next = ptr->next;
-      g_free(ptr);
-    }
-    d->data.data = NULL;
-    break;
-  default:
-    d->data.text = NULL;
-    break;
-  }
-}
-
 void
 update_viewer(struct obj_list_data *d)
 {
@@ -986,7 +682,7 @@ update(struct obj_list_data *d)
     return;
   }
 
-  parent = (Menulocal.single_window_mode) ? TopLevel : d->parent->Win;
+  parent = TopLevel;
 
   d->setup_dialog(d, sel, -1);
   d->select = sel;
@@ -1180,7 +876,11 @@ do_popup(GdkEventButton *event, struct obj_list_data *d)
   if (d->parent->type == TypeFileWin ||
       d->parent->type == TypeAxisWin ||
       d->parent->type == TypeMergeWin ||
-      d->parent->type == TypeLegendWin) {
+      d->parent->type == TypePathWin ||
+      d->parent->type == TypeRectWin ||
+      d->parent->type == TypeArcWin ||
+      d->parent->type == TypeMarkWin ||
+      d->parent->type == TypeTextWin) {
     d->select = list_store_get_selected_int(GTK_WIDGET(d->text), COL_ID);
   }
   gtk_menu_popup_at_pointer(GTK_MENU(d->popup), ((GdkEvent *)event));
@@ -1347,30 +1047,6 @@ ev_key_down(GtkWidget *w, GdkEvent *event, gpointer user_data)
   return TRUE;
 }
 
-static gboolean
-ev_sub_win_key_down(GtkWidget *w, GdkEvent *event, gpointer user_data)
-{
-  GdkEventKey *e;
-  struct SubWin *d;
-
-  d = user_data;
-
-  g_return_val_if_fail(w != NULL, FALSE);
-  g_return_val_if_fail(event != NULL, FALSE);
-
-  e = (GdkEventKey *)event;
-
-  switch (e->keyval) {
-  case GDK_KEY_w:
-    if (e->state & GDK_CONTROL_MASK) {
-      set_subwindow_state(d->type, SUBWIN_STATE_HIDE);
-      return TRUE;
-    }
-    break;
-  }
-  return FALSE;
-}
-
 #if WINDOWS
 #include <gdk/gdkwin32.h>
 
@@ -1386,6 +1062,20 @@ hide_minimize_menu_item(GtkWidget *widget, gpointer user_data)
 }
 #endif
 
+gboolean
+focus_in(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  gtk_grab_add(GTK_WIDGET(widget));
+  return FALSE;
+}
+
+gboolean
+focus_out(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  gtk_grab_remove(GTK_WIDGET(widget));
+  return FALSE;
+}
+
 static void
 swin_realized(GtkWidget *widget, gpointer user_data)
 {
@@ -1396,81 +1086,8 @@ swin_realized(GtkWidget *widget, gpointer user_data)
   ptr->update(ptr, TRUE, TRUE);
 }
 
-static GtkWidget *
-sub_window_create(struct SubWin *d, const char *title, GtkWidget *swin, const char *iconfile, const char *iconfile2)
-{
-  GtkWidget *dlg;
-  GdkPixbuf *icon;
-  GtkWindowGroup *group;
-
-  dlg = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  if (AccelGroup)
-    gtk_window_add_accel_group(GTK_WINDOW(dlg), AccelGroup);
-
-  d->Win = dlg;
-
-  if (iconfile) {
-    icon = gdk_pixbuf_new_from_resource(iconfile, NULL);
-
-    if (iconfile2) {
-      GList *tmp, *list = NULL;
-
-      if (icon) {
-	list = g_list_append(list, icon);
-      }
-
-      icon = gdk_pixbuf_new_from_resource(iconfile2, NULL);
-      if (icon) {
-	list = g_list_append(list, icon);
-      }
-
-      if (list) {
-	gtk_window_set_icon_list(GTK_WINDOW(dlg), list);
-	tmp = list;
-	while (tmp) {
-	  g_object_unref(tmp->data);
-	  tmp = tmp->next;
-	}
-	g_list_free(list);
-      }
-    } else {
-      if (icon) {
-	gtk_window_set_icon(GTK_WINDOW(dlg), icon);
-      }
-    }
-  }
-  if (title) {
-    gtk_window_set_title(GTK_WINDOW(dlg), title);
-  }
-
-  group = gtk_window_get_group(GTK_WINDOW(TopLevel));
-  gtk_window_group_add_window(group, GTK_WINDOW(dlg));
-  //  gtk_widget_set_parent_window(GTK_WIDGET(dlg), TopLevel);
-  //  gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
-  //  gtk_window_set_type_hint(GTK_WINDOW(dlg), GDK_WINDOW_TYPE_HINT_DIALOG);
-  gtk_window_set_type_hint(GTK_WINDOW(dlg), GDK_WINDOW_TYPE_HINT_UTILITY);
-  gtk_window_set_transient_for(GTK_WINDOW(dlg), GTK_WINDOW(TopLevel));
-  gtk_window_set_skip_taskbar_hint(GTK_WINDOW(dlg), TRUE);
-  gtk_window_set_skip_pager_hint(GTK_WINDOW(dlg), FALSE);
-  gtk_window_set_urgency_hint(GTK_WINDOW(dlg), FALSE);
-
-  gtk_container_add(GTK_CONTAINER(dlg), swin);
-
-#if WINDOWS
-  g_signal_connect(dlg, "realize", G_CALLBACK(hide_minimize_menu_item), NULL);
-#endif
-  g_signal_connect(dlg, "show", G_CALLBACK(cb_show), d);
-  g_signal_connect(dlg, "delete-event", G_CALLBACK(cb_del), d);
-  g_signal_connect(dlg, "destroy", G_CALLBACK(cb_destroy), d);
-  g_signal_connect(dlg, "key-press-event", G_CALLBACK(ev_sub_win_key_down), d);
-
-  gtk_widget_show_all(swin);
-
-  return dlg;
-}
-
 GtkWidget *
-text_sub_window_create(struct SubWin *d, const char *title, const char *icon, const char *icon2)
+text_sub_window_create(struct SubWin *d)
 {
   GtkWidget *view, *swin;
   GtkTextBuffer *buf;
@@ -1485,12 +1102,13 @@ text_sub_window_create(struct SubWin *d, const char *title, const char *icon, co
   swin = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_container_add(GTK_CONTAINER(swin), view);
+  d->Win = swin;
 
-  return sub_window_create(d, title, swin, icon, icon2);
+  return swin;
 }
 
 GtkWidget *
-label_sub_window_create(struct SubWin *d, const char *title, const char *icon, const char *icon2)
+label_sub_window_create(struct SubWin *d)
 {
   GtkWidget *label, *swin;
 
@@ -1510,8 +1128,9 @@ label_sub_window_create(struct SubWin *d, const char *title, const char *icon, c
 #else
   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(swin), label);
 #endif
+  d->Win = swin;
 
-  return sub_window_create(d, title, swin, icon, icon2);
+  return swin;
 }
 
 static gboolean
@@ -1535,7 +1154,6 @@ list_widget_create(struct SubWin *d, int lisu_num, n_list_store *list, int can_f
   data->can_focus = can_focus;
   data->list = list;
   data->list_col_num = lisu_num;
-  data->next = NULL;
   lstor = list_store_create(lisu_num, list);
   data->text = lstor;
 
@@ -1573,45 +1191,16 @@ list_widget_create(struct SubWin *d, int lisu_num, n_list_store *list, int can_f
 }
 
 GtkWidget *
-list_sub_window_create(struct SubWin *d, const char *title, int lisu_num, n_list_store *list, const char *icon, const char *icon2)
+list_sub_window_create(struct SubWin *d, int lisu_num, n_list_store *list)
 {
   GtkWidget *swin;
   struct obj_list_data *data;
 
   data = list_widget_create(d, lisu_num, list, d->type != TypeFileWin, &swin);
   d->data.data = data;
+  d->Win = swin;
 
-  return sub_window_create(d, title, swin, icon, icon2);
-}
-
-GtkWidget *
-tree_sub_window_create(struct SubWin *d, const char *title, int page_num, int *lisu_num, n_list_store **list, GtkWidget **icons, const char *icon, const char *icon2)
-{
-  GtkWidget *tab, *swin;
-  int i;
-  struct obj_list_data *data, *prev;
-  char *str;
-
-  tab = gtk_notebook_new();
-  gtk_notebook_popup_enable(GTK_NOTEBOOK(tab));
-
-  prev = NULL;
-  for (i = 0; i < page_num; i++) {
-    data = list_widget_create(d, lisu_num[i], list[i], TRUE, &swin);
-    if (prev) {
-      prev->next = data;
-    } else {
-      d->data.data = data;
-    }
-    gtk_notebook_append_page(GTK_NOTEBOOK(tab), swin, icons[i]);
-    str = gtk_widget_get_tooltip_text(icons[i]);
-    gtk_notebook_set_menu_label_text(GTK_NOTEBOOK(tab), swin, str);
-    g_free(str);
-    prev = data;
-  }
-  gtk_notebook_set_current_page(GTK_NOTEBOOK(tab), 0);
-
-  return sub_window_create(d, title, tab, icon, icon2);
+  return swin;
 }
 
 gboolean
@@ -1735,6 +1324,59 @@ ev_popup_menu(GtkWidget *w, gpointer client_data)
   return TRUE;
 }
 
+static int
+set_object_name(struct objlist *obj, int id)
+{
+  char *name, *new_name, buf[256];
+  int r;
+  getobj(obj, "name", id, 0, NULL, &name);
+  new_name = NULL;
+  snprintf(buf, sizeof(buf), "%s:%d:name", chkobjectname(obj), id);
+  r = DialogInput(TopLevel, _("Instance name"), buf, name, NULL, NULL, &new_name, NULL, NULL);
+  if (r != IDOK) {
+    return 0;
+  }
+  if (g_strcmp0(name, new_name) == 0) {
+    return 0;
+  }
+  if (new_name == NULL) {
+    putobj(obj, "name", id, new_name);
+    return 1;
+  }
+  g_strstrip(new_name);
+  if (new_name[0] == '\0') {
+    g_free(new_name);
+    new_name = NULL;
+  }
+  if (putobj(obj, "name", id, new_name) < 0) {
+    g_free(new_name);
+    return 0;
+  }
+  return 1;
+}
+
+void
+list_sub_window_object_name(GtkMenuItem *w, gpointer client_data)
+{
+  struct obj_list_data *d;
+
+  d = (struct obj_list_data*) client_data;
+  int sel, update, num;
+
+  if (Menulock || Globallock)
+    return;
+
+  sel = list_store_get_selected_int(GTK_WIDGET(d->text), COL_ID);
+  num = chkobjlastinst(d->obj);
+  if (sel < 0 || sel > num) {
+    return;
+  }
+  update = set_object_name(d->obj, sel);
+  if (update) {
+    set_graph_modified();
+  }
+}
+
 static GtkWidget *
 create_popup_menu_sub(struct obj_list_data *d, int top, struct subwin_popup_list *list)
 {
