@@ -25,10 +25,10 @@ struct subwin_popup_list {
   enum pop_up_menu_item_type type;
 };
 
-GtkWidget *label_sub_window_create(struct SubWin *d, const char *title, const char *icon, const char *icon2);
-GtkWidget *text_sub_window_create(struct SubWin *d, const char *title, const char *icon, const char *icon2);
-GtkWidget *list_sub_window_create(struct SubWin *d, const char *title, int lisu_num, n_list_store *list, const char *icon, const char *icon2);
-GtkWidget *tree_sub_window_create(struct SubWin *d, const char *title, int page_num, int *lisu_num, n_list_store **list, GtkWidget **icons, const char *icon, const char *icon2);
+GtkWidget *label_sub_window_create(struct SubWin *d);
+GtkWidget *text_sub_window_create(struct SubWin *d);
+GtkWidget *list_sub_window_create(struct SubWin *d, int lisu_num, n_list_store *list);
+GtkWidget *tree_sub_window_create(struct SubWin *d, int page_num, int *lisu_num, n_list_store **list, GtkWidget **icons);
 void sub_window_set_geometry(struct SubWin *d, int resize);
 void sub_window_save_geometry(struct SubWin *d);
 void sub_window_set_visibility(struct SubWin *d, int state);
@@ -44,6 +44,7 @@ gboolean list_sub_window_must_rebuild(struct obj_list_data *d);
 void list_sub_window_build(struct obj_list_data *d, list_sub_window_set_val_func func);
 void list_sub_window_set(struct obj_list_data *d, list_sub_window_set_val_func func);
 
+void list_sub_window_object_name(GtkMenuItem *w, gpointer client_data);
 void list_sub_window_delete(GtkMenuItem *item, gpointer user_data);
 void list_sub_window_copy(GtkMenuItem *item, gpointer user_data);
 void list_sub_window_move_top(GtkMenuItem *item, gpointer user_data);
