@@ -291,7 +291,7 @@ create_file_entry_with_cb(GCallback cb, gpointer data)
 
   w = create_text_entry(TRUE, TRUE);
 
-  gtk_entry_set_icon_from_icon_name(GTK_ENTRY(w), GTK_ENTRY_ICON_SECONDARY, "document-open");
+  gtk_entry_set_icon_from_icon_name(GTK_ENTRY(w), GTK_ENTRY_ICON_SECONDARY, "document-open-symbolic");
   g_signal_connect(w, "icon-release", cb, data);
 
   return w;
@@ -350,8 +350,8 @@ create_direction_entry(void)
 #if GTK_CHECK_VERSION(3, 12, 0)
   gtk_entry_set_max_width_chars(GTK_ENTRY(w), NUM_ENTRY_WIDTH);
 #endif
-  gtk_entry_set_icon_from_icon_name(GTK_ENTRY(w), GTK_ENTRY_ICON_SECONDARY, "go-up");
-  gtk_entry_set_icon_from_icon_name(GTK_ENTRY(w), GTK_ENTRY_ICON_PRIMARY, "go-down");
+  gtk_entry_set_icon_from_icon_name(GTK_ENTRY(w), GTK_ENTRY_ICON_SECONDARY, "go-up-symbolic");
+  gtk_entry_set_icon_from_icon_name(GTK_ENTRY(w), GTK_ENTRY_ICON_PRIMARY, "go-down-symbolic");
   g_signal_connect(w, "icon-release", G_CALLBACK(direction_icon_released), NULL);
 
   return w;
@@ -880,42 +880,6 @@ set_widget_font(GtkWidget *w, const char *font)
   }
 }
 
-#else  /* GTK_CHECK_VERSION(3, 16, 0) */
-
-#define LINE_NUMBER_R 0xCC00
-#define LINE_NUMBER_G 0xCC00
-#define LINE_NUMBER_B 0xCC00
-
-static void
-set_linumber_color(GtkWidget *w)
-{
-  GdkRGBA col;
-
-  col.red   = LINE_NUMBER_R * 1.0 / 0xFFFF;
-  col.green = LINE_NUMBER_G * 1.0 / 0xFFFF;
-  col.blue  = LINE_NUMBER_B * 1.0 / 0xFFFF;
-  col.alpha = 1.0;
-
-  gtk_widget_override_background_color(w, GTK_STATE_FLAG_NORMAL, &col);
-  gtk_widget_override_background_color(w, GTK_STATE_FLAG_ACTIVE, &col);
-  gtk_widget_override_background_color(w, GTK_STATE_FLAG_PRELIGHT, &col);
-  gtk_widget_override_background_color(w, GTK_STATE_FLAG_SELECTED, &col);
-  gtk_widget_override_background_color(w, GTK_STATE_FLAG_INSENSITIVE, &col);
-
-  col.red   = 0;
-  col.green = 0;
-  col.blue  = 0;
-  col.alpha = 1.0;
-
-  gtk_widget_override_color(w, GTK_STATE_FLAG_NORMAL, &col);
-  gtk_widget_override_color(w, GTK_STATE_FLAG_ACTIVE, &col);
-  gtk_widget_override_color(w, GTK_STATE_FLAG_PRELIGHT, &col);
-  gtk_widget_override_color(w, GTK_STATE_FLAG_SELECTED, &col);
-  gtk_widget_override_color(w, GTK_STATE_FLAG_INSENSITIVE, &col);
-
-  gtk_widget_set_sensitive(w, FALSE);
-}
-
 #endif	/* ! GTK_CHECK_VERSION(3, 16, 0) */
 
 GtkWidget *
@@ -1045,7 +1009,7 @@ select_obj_color(struct objlist *obj, int id, enum OBJ_FIELD_COLOR_TYPE type)
   color.blue = b / 255.0;
   color.alpha = a / 255.0;
 
-  dlg = gtk_color_chooser_dialog_new(title, GTK_WINDOW(NgraphApp.FileWin.Win));
+  dlg = gtk_color_chooser_dialog_new(title, GTK_WINDOW(TopLevel));
   if (Menulocal.use_custom_palette) {
     set_custom_palette(dlg);
   }
