@@ -39,6 +39,7 @@
 #include "gtk_subwin.h"
 #include "gtk_combo.h"
 #include "gtk_widget.h"
+#include "gtk_presettings.h"
 
 #include "x11bitmp.h"
 #include "x11gui.h"
@@ -108,6 +109,7 @@ static struct subwin_popup_list Popup_list[] = {
   {N_("_Focus"),      G_CALLBACK(list_sub_window_focus), NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
   {N_("_Clear"),      G_CALLBACK(axiswin_scale_clear), NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
   {N_("_Properties"), G_CALLBACK(list_sub_window_update), NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
+  {N_("_Instance name"), G_CALLBACK(list_sub_window_object_name), NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
   {NULL, NULL, NULL, POP_UP_MENU_ITEM_TYPE_SEPARATOR},
   {N_("_Top"),        G_CALLBACK(AxisWinAxisTop), NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
   {N_("_Up"),         G_CALLBACK(AxisWinAxisUp), NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
@@ -118,10 +120,10 @@ static struct subwin_popup_list Popup_list[] = {
 
 #define POPUP_ITEM_NUM (sizeof(Popup_list) / sizeof(*Popup_list) - 1)
 
-#define POPUP_ITEM_TOP 9
-#define POPUP_ITEM_UP 10
-#define POPUP_ITEM_DOWN 11
-#define POPUP_ITEM_BOTTOM 12
+#define POPUP_ITEM_TOP    10
+#define POPUP_ITEM_UP     11
+#define POPUP_ITEM_DOWN   12
+#define POPUP_ITEM_BOTTOM 13
 
 #define TITLE_BUF_SIZE 128
 
@@ -598,6 +600,7 @@ SectionDialogGrid(GtkWidget *w, gpointer client_data)
 	putobj(d->Obj2, "axis_y", *(d->IDG), ref);
       }
       create = TRUE;
+      presetting_set_obj_field(d->Obj2, *(d->IDG));
     }
   }
   if (*(d->IDG) >= 0) {
@@ -1941,8 +1944,8 @@ numbering_tab_create(GtkWidget *wi, struct AxisDialog *dd)
   d->add_plus = w;
   add_widget_to_table(table, w, NULL, FALSE, i++);
 
-  w = gtk_check_button_new_with_mnemonic(_("no _Zero"));
-  add_widget_to_table(table, w, NULL, FALSE, i++);
+  w = combo_box_create();
+  add_widget_to_table(table, w, _("_Zero:"), FALSE, i++);
   d->no_zero = w;
 
   w = create_text_entry(FALSE, TRUE);
@@ -2435,6 +2438,10 @@ CmAxisNewFrame(void *w, gpointer client_data)
   argv[1] = NULL;
   exeobj(obj, "default_grouping", idr, 1, argv);
   arraydel(&group);
+  presetting_set_obj_field(obj, idx);
+  presetting_set_obj_field(obj, idy);
+  presetting_set_obj_field(obj, idu);
+  presetting_set_obj_field(obj, idr);
   SectionDialog(&DlgSection, x, y, lenx, leny, obj, idx, idy, idu, idr, obj2,
 		&idg, FALSE);
   ret = DialogExecute(TopLevel, &DlgSection);
@@ -2499,6 +2506,11 @@ CmAxisNewSection(void *w, gpointer client_data)
       putobj(obj2, "axis_y", idg, ref);
     }
   }
+  presetting_set_obj_field(obj, idx);
+  presetting_set_obj_field(obj, idy);
+  presetting_set_obj_field(obj, idu);
+  presetting_set_obj_field(obj, idr);
+  presetting_set_obj_field(obj2, idg);
   SectionDialog(&DlgSection, x, y, lenx, leny, obj, idx, idy, idu, idr, obj2,
 		&idg, TRUE);
   ret = DialogExecute(TopLevel, &DlgSection);
@@ -2543,6 +2555,8 @@ CmAxisNewCross(void *w, gpointer client_data)
   argv[1] = NULL;
   exeobj(obj, "default_grouping", idy, 1, argv);
   arraydel(&group);
+  presetting_set_obj_field(obj, idx);
+  presetting_set_obj_field(obj, idy);
   CrossDialog(&DlgCross, x, y, lenx, leny, obj, idx, idy);
   ret = DialogExecute(TopLevel, &DlgCross);
   if (ret == IDCANCEL) {
@@ -2565,6 +2579,7 @@ CmAxisNewSingle(void *w, gpointer client_data)
     return;
   undo = axis_save_undo(UNDO_TYPE_CREATE);
   if ((id = newobj(obj)) >= 0) {
+    presetting_set_obj_field(obj, id);
     AxisDialog(NgraphApp.AxisWin.data.data, id, -1);
     ret = DialogExecute(TopLevel, &DlgAxis);
     if (ret == IDCANCEL) {
@@ -3273,7 +3288,9 @@ create_num_combo_item(GtkTreeStore *list, GtkTreeIter *parent, struct objlist *o
   add_text_combo_item_to_cbox(list, NULL, &iter, AXIS_COMBO_ITEM_NUM_COLOR, -1, _("Color"), TOGGLE_NONE, FALSE);
 
   add_bool_combo_item_to_cbox(list, NULL, &iter, AXIS_COMBO_ITEM_NUM_LOG, obj, "num_log_pow", id, _("Log power"));
-  add_bool_combo_item_to_cbox(list, NULL, &iter, AXIS_COMBO_ITEM_NUM_NO_ZERO, obj, "num_no_zero", id, _("No zero"));
+
+  add_text_combo_item_to_cbox(list, &child, &iter, -1, -1, _("Zero"), TOGGLE_NONE, FALSE);
+  add_enum_combo_item_to_cbox(list, NULL, &child, AXIS_COMBO_ITEM_NUM_NO_ZERO, obj, "num_no_zero", id);
 }
 
 static void
@@ -3430,9 +3447,11 @@ select_type(GtkComboBox *w, gpointer user_data)
     break;
   case AXIS_COMBO_ITEM_NUM_NO_ZERO:
     gtk_tree_model_get(GTK_TREE_MODEL(list), &iter, OBJECT_COLUMN_TYPE_TOGGLE, &active, -1);
+    if (type == enum_id) {
+      return;
+    }
     axis_save_undo(UNDO_TYPE_EDIT);
-    active = ! active;
-    putobj(d->obj, "num_no_zero", sel, &active);
+    putobj(d->obj, "num_no_zero", sel, &enum_id);
     break;
   case AXIS_COMBO_ITEM_NUM_COLOR:
     if (select_obj_color(obj, sel, OBJ_FIELD_COLOR_TYPE_AXIS_NUM)) {
@@ -3687,19 +3706,14 @@ axiswin_ev_key_down(GtkWidget *w, GdkEvent *event, gpointer user_data)
   return TRUE;
 }
 
-void
-AxisWinState(struct SubWin *d, int state)
+GtkWidget *
+dreate_axis_list(struct SubWin *d)
 {
   if (d->Win) {
-    sub_window_set_visibility(d, state);
-    return;
+    return d->Win;
   }
 
-  if (! state) {
-    return;
-  }
-
-  list_sub_window_create(d, "Axis Window", AXIS_WIN_COL_NUM, Alist, NGRAPH_AXISWIN_ICON_FILE, NGRAPH_AXISWIN_ICON48_FILE);
+  list_sub_window_create(d, AXIS_WIN_COL_NUM, Alist);
 
   d->data.data->update = AxisWinUpdate;
   d->data.data->setup_dialog = AxisDialog;
@@ -3729,4 +3743,6 @@ AxisWinState(struct SubWin *d, int state)
   gtk_tree_view_set_enable_search(GTK_TREE_VIEW(d->data.data->text), TRUE);
   gtk_tree_view_set_search_column(GTK_TREE_VIEW(d->data.data->text), AXIS_WIN_COL_NAME);
   tree_view_set_tooltip_column(GTK_TREE_VIEW(d->data.data->text), AXIS_WIN_COL_NAME);
+
+  return d->Win;
 }
