@@ -155,19 +155,17 @@ struct Viewer
   int allclear;
   int cx, cy;
   int ignoreredraw;
-  double vscroll, hscroll, Zoom;
-};
-
-enum subwin_state {
-  SUBWIN_STATE_SHOW,
-  SUBWIN_STATE_HIDE,
-  SUBWIN_STATE_TOGGLE,
+  double vscroll, hscroll, ZoomX, ZoomY;
 };
 
 enum SubWinType {
   TypeFileWin,
   TypeAxisWin,
-  TypeLegendWin,
+  TypePathWin,
+  TypeRectWin,
+  TypeArcWin,
+  TypeMarkWin,
+  TypeTextWin,
   TypeMergeWin,
   TypeCoordWin,
   TypeInfoWin,
@@ -190,24 +188,18 @@ struct obj_list_data
   struct SubWin *parent;
   n_list_store *list;
   int list_col_num;
-  struct obj_list_data *next;
 };
 
-struct SubWin;
-
 typedef void (* sub_window_state_func) (struct SubWin *d, int state);
 
 struct SubWin
 {
   enum SubWinType type;
   GtkWidget *Win;
-  GdkWindowState window_state;
-  int visible, action_widget_id;
   union {
     struct obj_list_data *data;
     GtkWidget *text;
   } data;
-  sub_window_state_func state_func;
 };
 
 #define MENU_HISTORY_NUM 10
@@ -224,7 +216,11 @@ struct NgraphApp
   struct Viewer Viewer;
   struct SubWin FileWin;
   struct SubWin AxisWin;
-  struct SubWin LegendWin;
+  struct SubWin PathWin;
+  struct SubWin RectWin;
+  struct SubWin ArcWin;
+  struct SubWin MarkWin;
+  struct SubWin TextWin;
   struct SubWin MergeWin;
   struct SubWin CoordWin;
   struct SubWin InfoWin;
@@ -306,8 +302,6 @@ void create_addin_menu(void);
 void create_recent_data_menu(void);
 void set_pointer_mode(int id);
 void set_toggle_action_widget_state(int id, int state);
-void set_subwindow_state(enum SubWinType id, enum subwin_state state);
-int get_subwindow_state(enum SubWinType id);
 void script_exec(GtkWidget *w, gpointer client_data);
 int toggle_view(int type, int state);
 void CmToggleSingleWindowMode(GtkCheckMenuItem *action, gpointer client_data);
