@@ -47,13 +47,14 @@
 #include "x11view.h"
 
 static n_list_store Mlist[] = {
-  {" ",        G_TYPE_BOOLEAN, TRUE, TRUE,  "hidden"},
-  {"#",        G_TYPE_INT,     TRUE, FALSE, "id"},
-  {N_("file"), G_TYPE_STRING,  TRUE, TRUE,  "file"},
-  {N_("top"),  G_TYPE_DOUBLE,  TRUE, TRUE,  "top_margin",  - SPIN_ENTRY_MAX, SPIN_ENTRY_MAX, 100, 1000},
-  {N_("left"), G_TYPE_DOUBLE,  TRUE, TRUE,  "left_margin", - SPIN_ENTRY_MAX, SPIN_ENTRY_MAX, 100, 1000},
-  {N_("zoom"), G_TYPE_DOUBLE,  TRUE, TRUE,  "zoom",                       0, SPIN_ENTRY_MAX, 100, 1000},
-  {"^#",       G_TYPE_INT,     TRUE, FALSE, "oid"},
+  {" ",          G_TYPE_BOOLEAN, TRUE, TRUE,  "hidden"},
+  {"#",          G_TYPE_INT,     TRUE, FALSE, "id"},
+  {N_("file"),   G_TYPE_STRING,  TRUE, TRUE,  "file"},
+  {N_("top"),    G_TYPE_DOUBLE,  TRUE, TRUE,  "top_margin",  - SPIN_ENTRY_MAX, SPIN_ENTRY_MAX, 100, 1000},
+  {N_("left"),   G_TYPE_DOUBLE,  TRUE, TRUE,  "left_margin", - SPIN_ENTRY_MAX, SPIN_ENTRY_MAX, 100, 1000},
+  {N_("zoom_x"), G_TYPE_DOUBLE,  TRUE, TRUE,  "zoom_x",                     0, SPIN_ENTRY_MAX, 100, 1000},
+  {N_("zoom_y"), G_TYPE_DOUBLE,  TRUE, TRUE,  "zoom_y",                     0, SPIN_ENTRY_MAX, 100, 1000},
+  {"^#",         G_TYPE_INT,     TRUE, FALSE, "oid"},
 };
 
 #define MERG_WIN_COL_NUM (sizeof(Mlist)/sizeof(*Mlist))
@@ -71,6 +72,7 @@ static struct subwin_popup_list Popup_list[] = {
   {NULL, NULL, NULL, POP_UP_MENU_ITEM_TYPE_SEPARATOR},
   {"_Focus",           G_CALLBACK(list_sub_window_focus), NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
   {N_("_Preferences"), G_CALLBACK(list_sub_window_update), NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
+  {N_("_Instance name"), G_CALLBACK(list_sub_window_object_name), NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
   {NULL, NULL, NULL, POP_UP_MENU_ITEM_TYPE_SEPARATOR},
   {N_("_Top"),    G_CALLBACK(list_sub_window_move_top), NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
   {N_("_Up"),     G_CALLBACK(list_sub_window_move_up), NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
@@ -80,10 +82,10 @@ static struct subwin_popup_list Popup_list[] = {
 };
 
 #define POPUP_ITEM_NUM (sizeof(Popup_list) / sizeof(*Popup_list) - 1)
-#define POPUP_ITEM_TOP 7
-#define POPUP_ITEM_UP 8
-#define POPUP_ITEM_DOWN 9
-#define POPUP_ITEM_BOTTOM 10
+#define POPUP_ITEM_TOP     8
+#define POPUP_ITEM_UP      9
+#define POPUP_ITEM_DOWN   10
+#define POPUP_ITEM_BOTTOM 11
 
 
 static void
@@ -95,7 +97,8 @@ MergeDialogSetupItem(struct MergeDialog *d, int file, int id)
   }
   SetWidgetFromObjField(d->topmargin, d->Obj, id, "top_margin");
   SetWidgetFromObjField(d->leftmargin, d->Obj, id, "left_margin");
-  SetWidgetFromObjField(d->zoom, d->Obj, id, "zoom");
+  SetWidgetFromObjField(d->zoom_x, d->Obj, id, "zoom_x");
+  SetWidgetFromObjField(d->zoom_y, d->Obj, id, "zoom_y");
 }
 
 static void
@@ -142,8 +145,12 @@ MergeDialogSetup(GtkWidget *wi, void *data, int makewidget)
     d->leftmargin = w;
 
     w = create_spin_entry_type(SPIN_BUTTON_TYPE_PERCENT, TRUE, TRUE);
-    add_widget_to_table(table, w, _("_Zoom:"), FALSE, i++);
-    d->zoom = w;
+    add_widget_to_table(table, w, _("zoom _X:"), FALSE, i++);
+    d->zoom_x = w;
+
+    w = create_spin_entry_type(SPIN_BUTTON_TYPE_PERCENT, TRUE, TRUE);
+    add_widget_to_table(table, w, _("zoom _Y:"), FALSE, i++);
+    d->zoom_y = w;
 
     frame = gtk_frame_new(NULL);
     gtk_container_add(GTK_CONTAINER(frame), table);
@@ -180,7 +187,9 @@ MergeDialogClose(GtkWidget *w, void *data)
     return;
   if (SetObjFieldFromWidget(d->leftmargin, d->Obj, d->Id, "left_margin"))
     return;
-  if (SetObjFieldFromWidget(d->zoom, d->Obj, d->Id, "zoom"))
+  if (SetObjFieldFromWidget(d->zoom_x, d->Obj, d->Id, "zoom_x"))
+    return;
+  if (SetObjFieldFromWidget(d->zoom_y, d->Obj, d->Id, "zoom_y"))
     return;
 
   d->ret = ret;
@@ -430,19 +439,14 @@ init_dnd(struct SubWin *d)
   g_signal_connect(widget, "drag-data-received", G_CALLBACK(drag_drop_cb), NULL);
 }
 
-void
-MergeWinState(struct SubWin *d, int state)
+GtkWidget *
+dreate_merge_list(struct SubWin *d)
 {
   if (d->Win) {
-    sub_window_set_visibility(d, state);
-    return;
+    return d->Win;
   }
 
-  if (! state) {
-    return;
-  }
-
-  list_sub_window_create(d, "Merge Window", MERG_WIN_COL_NUM, Mlist, NGRAPH_MERGEWIN_ICON_FILE, NGRAPH_MERGEWIN_ICON48_FILE);
+  list_sub_window_create(d, MERG_WIN_COL_NUM, Mlist);
 
   d->data.data->update = MergeWinUpdate;
   d->data.data->setup_dialog = MergeDialog;
@@ -456,4 +460,6 @@ MergeWinState(struct SubWin *d, int state)
   gtk_tree_view_set_enable_search(GTK_TREE_VIEW(d->data.data->text), TRUE);
   gtk_tree_view_set_search_column(GTK_TREE_VIEW(d->data.data->text), MERG_WIN_COL_FILE);
   tree_view_set_tooltip_column(GTK_TREE_VIEW(d->data.data->text), MERG_WIN_COL_FILE);
+
+  return d->Win;
 }
