@@ -67,14 +67,6 @@ DefaultDialogSetup(GtkWidget *wi, void *data, int makewidget)
   d = (struct DefaultDialog *) data;
 
   if (makewidget) {
-    w = gtk_check_button_new_with_mnemonic(_("_Geometry"));
-    d->geometry = w;
-    gtk_box_pack_start(GTK_BOX(d->vbox), w, FALSE, FALSE, 4);
-
-    w = gtk_check_button_new_with_mnemonic(_("_Child Geometry"));
-    d->child_geometry = w;
-    gtk_box_pack_start(GTK_BOX(d->vbox), w, FALSE, FALSE, 4);
-
     w = gtk_check_button_new_with_mnemonic(_("_Viewer"));
     d->viewer = w;
     gtk_box_pack_start(GTK_BOX(d->vbox), w, FALSE, FALSE, 4);
@@ -100,16 +92,11 @@ DefaultDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(d->vbox), w, FALSE, FALSE, 4);
     gtk_widget_show_all(GTK_WIDGET(d->vbox));
   }
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->geometry), FALSE);
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->child_geometry), FALSE);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->viewer), FALSE);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->external_viewer), FALSE);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->external_driver), FALSE);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->addin_script), FALSE);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->misc), FALSE);
-
-  gtk_widget_set_visible(d->geometry, ! Menulocal.single_window_mode);
-  gtk_widget_set_visible(d->child_geometry, ! Menulocal.single_window_mode);
 }
 
 static int
@@ -140,8 +127,6 @@ DefaultDialogClose(GtkWidget *win, void *data)
     GtkWidget *btn;
     enum SAVE_CONFIG_TYPE type;
   } btns[] = {
-    {NULL, SAVE_CONFIG_TYPE_GEOMETRY},
-    {NULL, SAVE_CONFIG_TYPE_CHILD_GEOMETRY},
     {NULL, SAVE_CONFIG_TYPE_VIEWER},
     {NULL, SAVE_CONFIG_TYPE_EXTERNAL_DRIVER},
     {NULL, SAVE_CONFIG_TYPE_ADDIN_SCRIPT},
@@ -158,14 +143,12 @@ DefaultDialogClose(GtkWidget *win, void *data)
   ret = d->ret;
   d->ret = IDLOOP;
 
-  btns[0].btn = d->geometry;
-  btns[1].btn = d->child_geometry;
-  btns[2].btn = d->viewer;
-  btns[3].btn = d->external_driver;
-  btns[4].btn = d->addin_script;
-  btns[5].btn = d->misc;
-  btns[6].btn = d->external_viewer;
-  btns[7].btn = d->fonts;
+  btns[0].btn = d->viewer;
+  btns[1].btn = d->external_driver;
+  btns[2].btn = d->addin_script;
+  btns[3].btn = d->misc;
+  btns[4].btn = d->external_viewer;
+  btns[5].btn = d->fonts;
 
   type = 0;
 
