@@ -662,7 +662,13 @@ CmOutputPrinter(int select_file, int show_dialog)
     opt = GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG;
   }
 
+#ifdef __WIN64__  
+  gtk_widget_set_visible(NgraphApp.Viewer.side_pane1, FALSE);
+#endif
   res = gtk_print_operation_run(print, opt, GTK_WINDOW(TopLevel), &error);
+#ifdef __WIN64__  
+  gtk_widget_set_visible(NgraphApp.Viewer.side_pane1, Menulocal.sidebar);
+#endif
 
   if (res == GTK_PRINT_OPERATION_RESULT_ERROR) {
     snprintf(buf, sizeof(buf), _("Printing error: %s"), error->message);
