@@ -1,24 +1,24 @@
-/* 
+/*
  * $Id: ioutil.c,v 1.25 2010-04-01 06:08:22 hito Exp $
- * 
+ *
  * This file is part of "Ngraph for X11".
- * 
+ *
  * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
- * 
+ *
  * "Ngraph for X11" is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * "Ngraph for X11" is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- * 
+ *
  */
 
 #include "common.h"
@@ -44,7 +44,7 @@
 #endif
 
 
-static void (* ShowProgressFunc)(int, char *, double) = NULL;
+static void (* ShowProgressFunc)(int, const char *, double) = NULL;
 
 #if EOF == -1
 static char C_type_buf[257];
@@ -62,10 +62,10 @@ char_type_buf_init(void)
 #define is_line_sep(ch) (C_type_buf[ch + 1])
 #endif
 
-void 
+void
 changefilename(char *name)
 {
-#ifdef WINDOWS
+#if WINDOWS
   int i;
 
   if (name == NULL) {
@@ -80,10 +80,10 @@ changefilename(char *name)
 #endif  /* WINDOWS */
 }
 
-void 
+void
 path_to_win(char *name)
 {
-#ifdef WINDOWS
+#if WINDOWS
   int i;
 
   if (name == NULL) {
@@ -98,7 +98,7 @@ path_to_win(char *name)
 #endif  /* WINDOWS */
 }
 
-static void 
+static void
 pathresolv(char *name)
 {
   int j,k;
@@ -171,7 +171,7 @@ getfullpath(const char *name)
   return s;
 }
 
-#ifdef WINDOWS
+#if WINDOWS
 static int
 getdisk(void)
 {
@@ -213,13 +213,13 @@ getrelativepath(const char *name)
     int i, j, top, depth;
 
     top = 0;
-#ifdef WINDOWS
+#if WINDOWS
     if (isalpha(utf8_name[0]) && name[1] == ':') {
       top = 2;
     }
 #endif	/* WINDOWS */
     if (
-#ifdef WINDOWS
+#if WINDOWS
 	name[top] == DIRSEP && (top != 2 || toupper(name[0]) - 'A' == getdisk())
 #else  /* WINDOWS */
 	name[0] == DIRSEP
@@ -232,7 +232,7 @@ getrelativepath(const char *name)
 	g_free(utf8_name);
 	return NULL;
       }
-#ifdef WINDOWS
+#if WINDOWS
       for (j = 2; cwd[j] != '\0'; j++) {
 	cwd[j - 2] = cwd[j];
       }
@@ -309,7 +309,7 @@ get_utf8_filename(const char *name)
   if (g_utf8_validate(name, -1, NULL)) {
     utf8_name = g_strdup(name);
   } else {
-#ifdef WINDOWS
+#if WINDOWS
     utf8_name = g_locale_to_utf8(name, -1, NULL, NULL, NULL);
 #else  /* WINDOWS */
     utf8_name = g_filename_to_utf8(name, -1, NULL, NULL, NULL);
@@ -330,7 +330,7 @@ get_localized_filename(const char *name)
     return NULL;
   }
 
-#ifdef WINDOWS
+#if WINDOWS
   if (g_utf8_validate(name, -1, NULL)) {
     localized_name = g_strdup(name);
   } else {
@@ -420,7 +420,7 @@ getfilename(const char *dir, const char *sep, const char *file)
   return s;
 }
 
-int 
+int
 findfilename(const char *dir, const char *sep, const char *file)
 {
   char *s;
@@ -490,7 +490,7 @@ nsearchpath(char *path,char *name,int shellscript)
   return NULL;
 }
 
-static int 
+static int
 nscandir(char *dir,char ***namelist, int (*compar)())
 {
   unsigned int i;
@@ -545,13 +545,13 @@ nscandir(char *dir,char ***namelist, int (*compar)())
   return alloc;
 }
 
-static int 
+static int
 nalphasort(char **a,char **b)
 {
   return strcmp(*a,*b);
 }
 
-static int 
+static int
 nglob2(char *path,int po,int *num,char ***list)
 {
   int i,j,p1,escape,scannum,len,err;
@@ -639,7 +639,7 @@ nglob2(char *path,int po,int *num,char ***list)
   return 0;
 }
 
-int 
+int
 nglob(char *path,char ***namelist)
 {
   int num;
@@ -660,7 +660,7 @@ nglob(char *path,char ***namelist)
   return num;
 }
 
-int 
+int
 fgetline(FILE *fp, char **buf)
 {
 /*
@@ -740,7 +740,7 @@ fgetline(FILE *fp, char **buf)
   }
 }
 
-int 
+int
 fgetnline(FILE *fp, char *buf, int len)
 {
 /*
@@ -765,7 +765,7 @@ fgetnline(FILE *fp, char *buf, int len)
   return 0;
 }
 
-int 
+int
 nfgetc(FILE *fp)
 {
   int ch;
@@ -796,7 +796,7 @@ nfopen(const char *filename, const char *mode)
   return fp;
 }
 
-int 
+int
 nisatty(int fd)
 {
   return isatty(fd);
@@ -862,7 +862,7 @@ nchdir(const gchar *path)
   return r;
 }
 
-int 
+int
 nopen(const char *path,int access,int mode)
 {
   int r;
@@ -882,31 +882,31 @@ nopen(const char *path,int access,int mode)
   return r;
 }
 
-void 
+void
 nclose(int fd)
 {
   close(fd);
 }
 
-void 
+void
 nlseek(int fd,long offset,int fromwhere)
 {
   lseek(fd,offset,fromwhere);
 }
 
-int 
+int
 nread(int fd,char *buf,unsigned len)
 {
   return read(fd,buf,len);
 }
 
-int 
+int
 nwrite(int fd,char *buf,unsigned len)
 {
   return write(fd,buf,len);
 }
 
-int 
+int
 nredirect(int fd,int newfd)
 {
   int savefd;
@@ -917,33 +917,33 @@ nredirect(int fd,int newfd)
   return savefd;
 }
 
-void 
+void
 nredirect2(int fd,int savefd)
 {
   dup2(savefd,fd);
   close(savefd);
 }
 
-int 
+int
 stdinfd(void)
 {
   return 0;
 }
 
-int 
+int
 stdoutfd(void)
 {
   return 1;
 }
 
-int 
+int
 stderrfd(void)
 {
   return 2;
 }
 
 void
-set_progress_func(void (* func)(int, char *, double))
+set_progress_func(void (* func)(int, const char *, double))
 {
   ShowProgressFunc = func;
 }
@@ -1014,7 +1014,7 @@ n_tmpfile(char **name)
     return NULL;
   }
 
-#ifndef WINDOWS
+#if ! WINDOWS
   if (*name) {
     g_unlink(*name);
   }
@@ -1024,7 +1024,7 @@ n_tmpfile(char **name)
   if (fp == NULL) {
     close(fd);
     if (*name) {
-#ifdef WINDOWS
+#if WINDOWS
       g_unlink(*name);
 #endif
       g_free(*name);
@@ -1043,7 +1043,7 @@ n_tmpfile_close(FILE *fp, char *name)
   }
 
   if (name) {
-#ifdef WINDOWS
+#if WINDOWS
     g_unlink(name);
 #endif
     g_free(name);
