@@ -1,24 +1,24 @@
-/* 
+/*
  * $Id: ioutil.h,v 1.9 2010-03-04 08:30:16 hito Exp $
- * 
+ *
  * This file is part of "Ngraph for X11".
- * 
+ *
  * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
- * 
+ *
  * "Ngraph for X11" is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * "Ngraph for X11" is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- * 
+ *
  */
 
 #ifndef IOUTIL_HEADER
@@ -36,7 +36,7 @@
 #define CONFTOP "/"
 #define NOHANDLE -1
 
-#ifndef WINDOWS
+#if ! WINDOWS
 #define PATHSEP ":"
 #define NFMODE (S_IRUSR | S_IWUSR)
 #define NFMODE_NORMAL_FILE (NFMODE | S_IRGRP | S_IROTH)
@@ -74,7 +74,7 @@ int nwrite(int fd,char *buf,unsigned len);
 int stdinfd(void);
 int stdoutfd(void);
 int stderrfd(void);
-void set_progress_func(void (* func)(int, char *, double));
+void set_progress_func(void (* func)(int, const char *, double));
 void set_progress(int pos, char *msg, double fraction);
 int n_mkstemp(const char *dir, char *templ, char **name);
 FILE *n_tmpfile(char **name);
