@@ -13,12 +13,21 @@ struct funcs {
 };
 
 static struct funcs FuncAry[] = {
+  {"ARRAY_AVERAGE", {1, 1, 0, math_func_array_average, NULL, NULL, NULL, NULL}},
+  {"ARRAY_COMPACT", {1, 1, 0, math_func_array_compact, NULL, NULL, NULL, NULL}},
+  {"ARRAY_STDEVP", {1, 1, 0, math_func_array_stdevp, NULL, NULL, NULL, NULL}},
+  {"ARRAY_CLEAR", {1, 1, 0, math_func_array_clear, NULL, NULL, NULL, NULL}},
+  {"ARRAY_SUMSQ", {1, 1, 0, math_func_array_sumsq, NULL, NULL, NULL, NULL}},
+  {"ARRAY_STDEV", {1, 1, 0, math_func_array_stdev, NULL, NULL, NULL, NULL}},
 #ifdef HAVE_LIBGSL
   {"ZETAM1_INT", {1, 0, 0, math_func_zetam1_int, NULL, NULL, NULL, NULL}},
 #else
   {NULL, {0, 0, 0, NULL, NULL, NULL, NULL, NULL}},
 #endif
   {"MJD2MONTH", {1, 0, 0, math_func_mjd2month, NULL, NULL, NULL, NULL}},
+  {"ARRAY_SUM", {1, 1, 0, math_func_array_sum, NULL, NULL, NULL, NULL}},
+  {"ARRAY_MIN", {1, 1, 0, math_func_array_min, NULL, NULL, NULL, NULL}},
+  {"ARRAY_MAX", {1, 1, 0, math_func_array_max, NULL, NULL, NULL, NULL}},
   {"UNIX2MJD", {1, 0, 0, math_func_unix2mjd, NULL, NULL, NULL, NULL}},
   {"MJD2YEAR", {1, 0, 0, math_func_mjd2year, NULL, NULL, NULL, NULL}},
   {"MJD2WDAY", {1, 0, 0, math_func_mjd2wday, NULL, NULL, NULL, NULL}},
@@ -62,11 +71,13 @@ static struct funcs FuncAry[] = {
   {"GAMMA", {1, 0, 0, math_func_gamma, NULL, NULL, NULL, NULL}},
   {"ICGAM", {2, 0, 0, math_func_icgam, NULL, NULL, NULL, NULL}},
   {"RSORT", {1, 1, 0, math_func_rsort, NULL, NULL, NULL, NULL}},
+  {"ARRAY", {-2, 1, 0, math_func_array, NULL, NULL, NULL, NULL}},
   {"ISNAN", {1, 0, 0, math_func_isnan, NULL, NULL, NULL, NULL}},
   {"PROGN", {-1, 0, 0, math_func_progn, NULL, NULL, NULL, NULL}},
   {"PROG1", {-1, 0, 0, math_func_prog1, NULL, NULL, NULL, NULL}},
   {"PROG2", {-1, 0, 0, math_func_prog2, NULL, NULL, NULL, NULL}},
   {"SHIFT", {1, 0, 0, math_func_shift, NULL, NULL, NULL, NULL}},
+  {"SUMSQ", {-1, 0, 0, math_func_sumsq, NULL, NULL, NULL, NULL}},
   {"SIGN", {1, 0, 0, math_func_sign, NULL, NULL, NULL, NULL}},
   {"FRAC", {1, 0, 0, math_func_frac, NULL, NULL, NULL, NULL}},
   {"SQRT", {1, 0, 0, math_func_sqrt, NULL, NULL, NULL, NULL}},
@@ -149,7 +160,8 @@ static struct funcs FuncAry[] = {
   {"LT", {3, 0, 0, math_func_lt, NULL, NULL, NULL, NULL}},
   {"OR", {2, 0, 0, math_func_or, NULL, NULL, NULL, NULL}},
   {"RM", {1, 1, 0, math_func_rm, NULL, NULL, NULL, NULL}},
-  {"CM", {1, 0, 0, math_func_cm, NULL, NULL, NULL, NULL}},
+  {"CM", {2, 1, 0, math_func_cm, NULL, NULL, NULL, NULL}},
+  {"AM", {1, 1, 0, math_func_am, NULL, NULL, NULL, NULL}},
   {"IF", {3, 0, 0, math_func_if, NULL, NULL, NULL, NULL}},
 #ifdef HAVE_LIBGSL
   {"IN", {2, 0, 0, math_func_in, NULL, NULL, NULL, NULL}},
@@ -185,7 +197,106 @@ math_add_basic_function(MathEquation *eq) {
       continue;
     }
     switch (i) {
-    case 12:  /*  UNSHIFT  */
+    case 0:  /*  ARRAY_AVERAGE  */
+      if (FuncAry[i].prm.arg_type) {
+        break;
+      }
+      ptr = g_malloc(sizeof(enum MATH_FUNCTION_ARG_TYPE) * 1);
+      if (ptr == NULL) {
+        return 1;
+      }
+      ptr[0] = MATH_FUNCTION_ARG_TYPE_ARRAY;
+      FuncAry[i].prm.arg_type = ptr;
+      break;
+    case 1:  /*  ARRAY_COMPACT  */
+      if (FuncAry[i].prm.arg_type) {
+        break;
+      }
+      ptr = g_malloc(sizeof(enum MATH_FUNCTION_ARG_TYPE) * 1);
+      if (ptr == NULL) {
+        return 1;
+      }
+      ptr[0] = MATH_FUNCTION_ARG_TYPE_ARRAY;
+      FuncAry[i].prm.arg_type = ptr;
+      break;
+    case 2:  /*  ARRAY_STDEVP  */
+      if (FuncAry[i].prm.arg_type) {
+        break;
+      }
+      ptr = g_malloc(sizeof(enum MATH_FUNCTION_ARG_TYPE) * 1);
+      if (ptr == NULL) {
+        return 1;
+      }
+      ptr[0] = MATH_FUNCTION_ARG_TYPE_ARRAY;
+      FuncAry[i].prm.arg_type = ptr;
+      break;
+    case 3:  /*  ARRAY_CLEAR  */
+      if (FuncAry[i].prm.arg_type) {
+        break;
+      }
+      ptr = g_malloc(sizeof(enum MATH_FUNCTION_ARG_TYPE) * 1);
+      if (ptr == NULL) {
+        return 1;
+      }
+      ptr[0] = MATH_FUNCTION_ARG_TYPE_ARRAY;
+      FuncAry[i].prm.arg_type = ptr;
+      break;
+    case 4:  /*  ARRAY_SUMSQ  */
+      if (FuncAry[i].prm.arg_type) {
+        break;
+      }
+      ptr = g_malloc(sizeof(enum MATH_FUNCTION_ARG_TYPE) * 1);
+      if (ptr == NULL) {
+        return 1;
+      }
+      ptr[0] = MATH_FUNCTION_ARG_TYPE_ARRAY;
+      FuncAry[i].prm.arg_type = ptr;
+      break;
+    case 5:  /*  ARRAY_STDEV  */
+      if (FuncAry[i].prm.arg_type) {
+        break;
+      }
+      ptr = g_malloc(sizeof(enum MATH_FUNCTION_ARG_TYPE) * 1);
+      if (ptr == NULL) {
+        return 1;
+      }
+      ptr[0] = MATH_FUNCTION_ARG_TYPE_ARRAY;
+      FuncAry[i].prm.arg_type = ptr;
+      break;
+    case 8:  /*  ARRAY_SUM  */
+      if (FuncAry[i].prm.arg_type) {
+        break;
+      }
+      ptr = g_malloc(sizeof(enum MATH_FUNCTION_ARG_TYPE) * 1);
+      if (ptr == NULL) {
+        return 1;
+      }
+      ptr[0] = MATH_FUNCTION_ARG_TYPE_ARRAY;
+      FuncAry[i].prm.arg_type = ptr;
+      break;
+    case 9:  /*  ARRAY_MIN  */
+      if (FuncAry[i].prm.arg_type) {
+        break;
+      }
+      ptr = g_malloc(sizeof(enum MATH_FUNCTION_ARG_TYPE) * 1);
+      if (ptr == NULL) {
+        return 1;
+      }
+      ptr[0] = MATH_FUNCTION_ARG_TYPE_ARRAY;
+      FuncAry[i].prm.arg_type = ptr;
+      break;
+    case 10:  /*  ARRAY_MAX  */
+      if (FuncAry[i].prm.arg_type) {
+        break;
+      }
+      ptr = g_malloc(sizeof(enum MATH_FUNCTION_ARG_TYPE) * 1);
+      if (ptr == NULL) {
+        return 1;
+      }
+      ptr[0] = MATH_FUNCTION_ARG_TYPE_ARRAY;
+      FuncAry[i].prm.arg_type = ptr;
+      break;
+    case 21:  /*  UNSHIFT  */
       if (FuncAry[i].prm.arg_type) {
         break;
       }
@@ -197,7 +308,7 @@ math_add_basic_function(MathEquation *eq) {
       ptr[1] = MATH_FUNCTION_ARG_TYPE_DOUBLE;
       FuncAry[i].prm.arg_type = ptr;
       break;
-    case 14:  /*  UNLESS  */
+    case 23:  /*  UNLESS  */
       if (FuncAry[i].prm.arg_type) {
         break;
       }
@@ -210,7 +321,7 @@ math_add_basic_function(MathEquation *eq) {
       ptr[2] = MATH_FUNCTION_ARG_TYPE_PROC;
       FuncAry[i].prm.arg_type = ptr;
       break;
-    case 28:  /*  RSORT  */
+    case 37:  /*  RSORT  */
       if (FuncAry[i].prm.arg_type) {
         break;
       }
@@ -221,7 +332,7 @@ math_add_basic_function(MathEquation *eq) {
       ptr[0] = MATH_FUNCTION_ARG_TYPE_ARRAY;
       FuncAry[i].prm.arg_type = ptr;
       break;
-    case 33:  /*  SHIFT  */
+    case 38:  /*  ARRAY  */
       if (FuncAry[i].prm.arg_type) {
         break;
       }
@@ -232,7 +343,7 @@ math_add_basic_function(MathEquation *eq) {
       ptr[0] = MATH_FUNCTION_ARG_TYPE_ARRAY;
       FuncAry[i].prm.arg_type = ptr;
       break;
-    case 47:  /*  SORT  */
+    case 43:  /*  SHIFT  */
       if (FuncAry[i].prm.arg_type) {
         break;
       }
@@ -243,7 +354,7 @@ math_add_basic_function(MathEquation *eq) {
       ptr[0] = MATH_FUNCTION_ARG_TYPE_ARRAY;
       FuncAry[i].prm.arg_type = ptr;
       break;
-    case 48:  /*  SIZE  */
+    case 58:  /*  SORT  */
       if (FuncAry[i].prm.arg_type) {
         break;
       }
@@ -254,7 +365,18 @@ math_add_basic_function(MathEquation *eq) {
       ptr[0] = MATH_FUNCTION_ARG_TYPE_ARRAY;
       FuncAry[i].prm.arg_type = ptr;
       break;
-    case 50:  /*  PUSH  */
+    case 59:  /*  SIZE  */
+      if (FuncAry[i].prm.arg_type) {
+        break;
+      }
+      ptr = g_malloc(sizeof(enum MATH_FUNCTION_ARG_TYPE) * 1);
+      if (ptr == NULL) {
+        return 1;
+      }
+      ptr[0] = MATH_FUNCTION_ARG_TYPE_ARRAY;
+      FuncAry[i].prm.arg_type = ptr;
+      break;
+    case 61:  /*  PUSH  */
       if (FuncAry[i].prm.arg_type) {
         break;
       }
@@ -266,7 +388,7 @@ math_add_basic_function(MathEquation *eq) {
       ptr[1] = MATH_FUNCTION_ARG_TYPE_DOUBLE;
       FuncAry[i].prm.arg_type = ptr;
       break;
-    case 70:  /*  FOR  */
+    case 81:  /*  FOR  */
       if (FuncAry[i].prm.arg_type) {
         break;
       }
@@ -281,7 +403,7 @@ math_add_basic_function(MathEquation *eq) {
       ptr[4] = MATH_FUNCTION_ARG_TYPE_PROC;
       FuncAry[i].prm.arg_type = ptr;
       break;
-    case 77:  /*  POP  */
+    case 88:  /*  POP  */
       if (FuncAry[i].prm.arg_type) {
         break;
       }
@@ -292,7 +414,18 @@ math_add_basic_function(MathEquation *eq) {
       ptr[0] = MATH_FUNCTION_ARG_TYPE_ARRAY;
       FuncAry[i].prm.arg_type = ptr;
       break;
-    case 93:  /*  IF  */
+    case 104:  /*  AM  */
+      if (FuncAry[i].prm.arg_type) {
+        break;
+      }
+      ptr = g_malloc(sizeof(enum MATH_FUNCTION_ARG_TYPE) * 1);
+      if (ptr == NULL) {
+        return 1;
+      }
+      ptr[0] = MATH_FUNCTION_ARG_TYPE_ARRAY;
+      FuncAry[i].prm.arg_type = ptr;
+      break;
+    case 105:  /*  IF  */
       if (FuncAry[i].prm.arg_type) {
         break;
       }
