@@ -1,6 +1,6 @@
-/* 
+/*
  * $Id: math_parser.c,v 1.19 2010-03-04 08:30:17 hito Exp $
- * 
+ *
  */
 
 #include "config.h"
@@ -13,8 +13,8 @@
 
 #include "object.h"
 
-#include "math_scanner.h"
 #include "math_equation.h"
+#include "math_scanner.h"
 #include "math_function.h"
 #include "parse_bin_expression.h"
 
@@ -26,13 +26,12 @@ MathValue MATH_VALUE_ZERO = {0.0, MATH_VALUE_NORMAL};
 
 static struct math_token *st_look_ahead_token = NULL;
 
-static MathExpression * parse_expression(const char **str, MathEquation *eq, int *err);
-static MathExpression * parse_expression_list(const char **str, MathEquation *eq, int inside_block, int *err);
-static MathExpression * parse_unary_expression(const char **str, MathEquation *eq, int *err);
-
+static MathExpression * parse_expression(struct math_string *mstr, MathEquation *eq, int *err);
+static MathExpression * parse_expression_list(struct math_string *mstr, MathEquation *eq, int inside_block, int *err);
+static MathExpression * parse_unary_expression(struct math_string *mstr, MathEquation *eq, int *err);
 
 static struct math_token *
-my_get_token(const const char **str)
+my_get_token(struct math_string *str)
 {
   struct math_token *token;
 
@@ -41,7 +40,7 @@ my_get_token(const const char **str)
     st_look_ahead_token = token->next;
     token->next = NULL;
   } else {
-    token = math_scanner_get_token(*str, str);
+    token = math_scanner_get_token(str);
   }
 
   return token;
@@ -55,7 +54,7 @@ unget_token(struct math_token *token)
 }
 
 static MathExpression *
-parse_array_expression(const char **str, MathEquation *eq, const char *name, int *err)
+parse_array_expression(struct math_string *str, MathEquation *eq, const char *name, int *err)
 {
   struct math_token *token;
   MathExpression *operand, *exp;
@@ -80,7 +79,7 @@ parse_array_expression(const char **str, MathEquation *eq, const char *name, int
 
   if (token->type != MATH_TOKEN_TYPE_RB) {
     *err = MATH_ERROR_MISS_RB;
-    math_equation_set_parse_error(eq, token->ptr);
+    math_equation_set_parse_error(eq, token->ptr, str);
     math_scanner_free_token(token);
     math_expression_free(operand);
     return NULL;
@@ -98,7 +97,7 @@ parse_array_expression(const char **str, MathEquation *eq, const char *name, int
 }
 
 static MathExpression *
-parse_primary_expression(const char **str, MathEquation *eq, int *err)
+parse_primary_expression(struct math_string *str, MathEquation *eq, int *err)
 {
   struct math_token *token, *token2;
   MathExpression *exp;
@@ -131,16 +130,16 @@ parse_primary_expression(const char **str, MathEquation *eq, int *err)
 	if (token->data.sym[0] == '%') {
 	  exp = math_parameter_expression_new(eq, token->data.sym, err);
 	  if (exp == NULL) {
-	    math_equation_set_parse_error(eq, token->ptr);
+	    math_equation_set_parse_error(eq, token->ptr, str);
 	    math_scanner_free_token(token);
 	    return NULL;
 	  }
-#if 1				/* length of a variable expression is 1. */
-	} else if (strlen(token->data.sym) == 1) {
+#if 0				/* length of a variable expression is less than 3 other than start with "_". */
+	} else if (token->data.sym[0] == '_' || strlen(token->data.sym) < 3) {
 	  exp = math_variable_expression_new(eq, token->data.sym, err);
 	} else {
 	  *err = MATH_ERROR_UNEXP_TOKEN;
-	  math_equation_set_parse_error(eq, token->ptr);
+	  math_equation_set_parse_error(eq, token->ptr, str);
 	  math_scanner_free_token(token);
 	  return NULL;
 #else				/* length of a variable expression is not fixed. */
@@ -166,7 +165,7 @@ parse_primary_expression(const char **str, MathEquation *eq, int *err)
 
     if (token->type != MATH_TOKEN_TYPE_RP) {
       *err = MATH_ERROR_MISS_RP;
-      math_equation_set_parse_error(eq, token->ptr);
+      math_equation_set_parse_error(eq, token->ptr, str);
       math_scanner_free_token(token);
       math_expression_free(exp);
       return NULL;
@@ -178,7 +177,7 @@ parse_primary_expression(const char **str, MathEquation *eq, int *err)
     return NULL;
   default:
     *err = MATH_ERROR_UNEXP_TOKEN;
-    math_equation_set_parse_error(eq, token->ptr);
+    math_equation_set_parse_error(eq, token->ptr, str);
     math_scanner_free_token(token);
     return NULL;
   }
@@ -199,16 +198,18 @@ free_arg_list(MathExpression **argv)
 }
 
 static MathExpression *
-get_argument(const char **str, MathEquation *eq, struct math_function_parameter *fprm, int i, int *err)
+get_argument(struct math_string *str, MathEquation *eq, struct math_function_parameter *fprm, int i, int *err)
 {
   struct math_token *token;
   MathExpression *exp;
+  int argc;
 
-  if (fprm->arg_type && i < fprm->argc && fprm->arg_type[i] == MATH_FUNCTION_ARG_TYPE_ARRAY) {
+  argc = math_function_get_arg_type_num(fprm);
+  if (fprm->arg_type && i < argc && fprm->arg_type[i] == MATH_FUNCTION_ARG_TYPE_ARRAY) {
     token = my_get_token(str);
     if (token->type != MATH_TOKEN_TYPE_SYMBOL) {
       *err = MATH_ERROR_INVALID_FDEF;
-      math_equation_set_parse_error(eq, token->ptr);
+      math_equation_set_parse_error(eq, token->ptr, str);
       math_scanner_free_token(token);
       /* invalid argument */
       return NULL;
@@ -223,7 +224,7 @@ get_argument(const char **str, MathEquation *eq, struct math_function_parameter
 }
 
 static int
-parse_argument_list(const char **str, MathEquation *eq, struct math_function_parameter *fprm, MathExpression ***buf, int argc, int *err)
+parse_argument_list(struct math_string *str, MathEquation *eq, struct math_function_parameter *fprm, MathExpression ***buf, int argc, int *err)
 {
   struct math_token *token;
   int i, n;
@@ -292,7 +293,7 @@ parse_argument_list(const char **str, MathEquation *eq, struct math_function_par
 }
 
 static MathExpression *
-create_math_func(const char **str, MathEquation *eq, struct math_token *name, int *err)
+create_math_func(struct math_string *str, MathEquation *eq, struct math_token *name, int *err)
 {
   struct math_function_parameter *fprm;
   int i, arg_max, argc, pos_id;
@@ -302,7 +303,7 @@ create_math_func(const char **str, MathEquation *eq, struct math_token *name, in
   fprm = math_equation_get_func(eq, name->data.sym);
   if (fprm == NULL) {
     *err = MATH_ERROR_UNKNOWN_FUNC;
-    math_equation_set_parse_error(eq, name->ptr);
+    math_equation_set_parse_error(eq, name->ptr, str);
     return NULL;
   }
 
@@ -323,7 +324,7 @@ create_math_func(const char **str, MathEquation *eq, struct math_token *name, in
   token = my_get_token(str);
   if (token->type != MATH_TOKEN_TYPE_RP) {
     *err = MATH_ERROR_MISS_RP;
-    math_equation_set_parse_error(eq, token->ptr);
+    math_equation_set_parse_error(eq, token->ptr, str);
     free_arg_list(argv);
     g_free(argv);
     math_scanner_free_token(token);
@@ -369,7 +370,7 @@ create_math_func(const char **str, MathEquation *eq, struct math_token *name, in
 }
 
 static MathExpression *
-parse_func_expression(const char **str, MathEquation *eq, int *err)
+parse_func_expression(struct math_string *str, MathEquation *eq, int *err)
 {
   struct math_token *token1, *token2;
   MathExpression *exp;
@@ -408,7 +409,7 @@ parse_func_expression(const char **str, MathEquation *eq, int *err)
 
 
 static MathExpression *
-parse_factorial_expression(const char **str, MathEquation *eq, int *err)
+parse_factorial_expression(struct math_string *str, MathEquation *eq, int *err)
 {
   struct math_token *token;
   MathExpression *operand, *exp;
@@ -454,7 +455,7 @@ parse_factorial_expression(const char **str, MathEquation *eq, int *err)
 }
 
 static MathExpression *
-parse_power_expression(const char **str, MathEquation *eq, int *err)
+parse_power_expression(struct math_string *str, MathEquation *eq, int *err)
 {
   struct math_token *token;
   MathExpression *left, *right, *exp;
@@ -505,7 +506,7 @@ parse_power_expression(const char **str, MathEquation *eq, int *err)
 }
 
 static MathExpression *
-parse_unary_expression(const char **str, MathEquation *eq, int *err)
+parse_unary_expression(struct math_string *str, MathEquation *eq, int *err)
 {
   struct math_token *token;
   MathExpression *exp, *operand;
@@ -537,7 +538,7 @@ parse_unary_expression(const char **str, MathEquation *eq, int *err)
       break;
     default:
       *err = MATH_ERROR_UNEXP_OPE;
-      math_equation_set_parse_error(eq, token->ptr);
+      math_equation_set_parse_error(eq, token->ptr, str);
       math_scanner_free_token(token);
       return NULL;
     }
@@ -660,7 +661,7 @@ create_variable_assign_expression(MathEquation *eq, enum MATH_OPERATOR_TYPE op,
 }
 
 static MathExpression *
-parse_assign_expression(const char **str, MathEquation *eq, enum MATH_OPERATOR_TYPE op, MathExpression *lexp, int *err)
+parse_assign_expression(struct math_string *str, MathEquation *eq, enum MATH_OPERATOR_TYPE op, MathExpression *lexp, int *err)
 {
   MathExpression *exp, *rexp;
 
@@ -684,7 +685,7 @@ parse_assign_expression(const char **str, MathEquation *eq, enum MATH_OPERATOR_T
 }
 
 static int
-parse_parameter_list(const char **str, MathEquation *eq, MathExpression *func, int *err)
+parse_parameter_list(struct math_string *str, MathEquation *eq, MathExpression *func, int *err)
 {
   struct math_token *token;
   enum MATH_FUNCTION_ARG_TYPE type;
@@ -762,7 +763,7 @@ parse_parameter_list(const char **str, MathEquation *eq, MathExpression *func, i
 }
 
 static MathExpression *
-parse_block_expression(const char **str, MathEquation *eq, int *err)
+parse_block_expression(struct math_string *str, MathEquation *eq, int *err)
 {
   struct math_token *token;
   MathExpression *exp;
@@ -821,7 +822,7 @@ free_func_prm(struct math_function_parameter *prm)
 }
 
 MathExpression *
-parse_func_def_expression(const char **str, MathEquation *eq, int *err)
+parse_func_def_expression(struct math_string *str, MathEquation *eq, int *err)
 {
   struct math_token *fname;
   MathExpression *exp, *block;
@@ -883,7 +884,7 @@ parse_func_def_expression(const char **str, MathEquation *eq, int *err)
 }
 
 MathExpression *
-parse_const_def_expression(const char **str, MathEquation *eq, int *err)
+parse_const_def_expression(struct math_string *str, MathEquation *eq, int *err)
 {
   struct math_token *cname, *token;
   MathExpression *exp, *cdef;
@@ -901,7 +902,7 @@ parse_const_def_expression(const char **str, MathEquation *eq, int *err)
 
   if (cname->type != MATH_TOKEN_TYPE_SYMBOL) {
     *err = MATH_ERROR_UNEXP_TOKEN;
-    math_equation_set_parse_error(eq, cname->ptr);
+    math_equation_set_parse_error(eq, cname->ptr, str);
     math_scanner_free_token(cname);
     return NULL;
   }
@@ -918,7 +919,7 @@ parse_const_def_expression(const char **str, MathEquation *eq, int *err)
       *err = MATH_ERROR_EOEQ;
     } else {
       *err = MATH_ERROR_UNEXP_TOKEN;
-      math_equation_set_parse_error(eq, token->ptr);
+      math_equation_set_parse_error(eq, token->ptr, str);
     }
     math_scanner_free_token(token);
     math_scanner_free_token(cname);
@@ -945,7 +946,7 @@ parse_const_def_expression(const char **str, MathEquation *eq, int *err)
 }
 
 static MathExpression *
-parse_expression(const char **str, MathEquation *eq, int *err)
+parse_expression(struct math_string *str, MathEquation *eq, int *err)
 {
   struct math_token *token;
   MathExpression *exp;
@@ -984,7 +985,7 @@ parse_expression(const char **str, MathEquation *eq, int *err)
       break;
     default:
       *err = MATH_ERROR_UNEXP_OPE;
-      math_equation_set_parse_error(eq, token->ptr);
+      math_equation_set_parse_error(eq, token->ptr, str);
       math_scanner_free_token(token);
       math_expression_free(exp);
       return NULL;
@@ -999,7 +1000,7 @@ parse_expression(const char **str, MathEquation *eq, int *err)
     break;
   default:
     *err = MATH_ERROR_UNEXP_TOKEN;
-    math_equation_set_parse_error(eq, token->ptr);
+    math_equation_set_parse_error(eq, token->ptr, str);
     math_scanner_free_token(token);
     math_expression_free(exp);
     return NULL;
@@ -1011,7 +1012,7 @@ parse_expression(const char **str, MathEquation *eq, int *err)
 }
 
 static MathExpression *
-parse_expression_list(const char ** str, MathEquation *eq, int inside_block, int *err)
+parse_expression_list(struct math_string *str, MathEquation *eq, int inside_block, int *err)
 {
   struct math_token *token;
   MathExpression *exp, *prev, *top;
@@ -1029,7 +1030,7 @@ parse_expression_list(const char ** str, MathEquation *eq, int inside_block, int
     case MATH_TOKEN_TYPE_DEF:
       if (inside_block) {
 	*err = MATH_ERROR_FDEF_NEST;
-	math_equation_set_parse_error(eq, token->ptr);
+	math_equation_set_parse_error(eq, token->ptr, str);
 	math_scanner_free_token(token);
 	math_expression_free(top);
 	return NULL;
@@ -1044,7 +1045,7 @@ parse_expression_list(const char ** str, MathEquation *eq, int inside_block, int
     case MATH_TOKEN_TYPE_CONST:
       if (inside_block) {
 	*err = MATH_ERROR_INVALID_CDEF;
-	math_equation_set_parse_error(eq, token->ptr);
+	math_equation_set_parse_error(eq, token->ptr, str);
 	math_scanner_free_token(token);
 	math_expression_free(top);
 	return NULL;
@@ -1057,10 +1058,10 @@ parse_expression_list(const char ** str, MathEquation *eq, int inside_block, int
       }
       continue;
     case MATH_TOKEN_TYPE_EOEQ:
-      if (*str[0] == '\0') {
+      if (str->cur[0] == '\0') {
 	if (inside_block) {
 	  *err = MATH_ERROR_MISS_RC;
-	  math_equation_set_parse_error(eq, token->ptr);
+	  math_equation_set_parse_error(eq, token->ptr, str);
 	  math_scanner_free_token(token);
 	  math_expression_free(top);
 	  return NULL;
@@ -1076,7 +1077,7 @@ parse_expression_list(const char ** str, MathEquation *eq, int inside_block, int
 	goto End;
       }
       *err = MATH_ERROR_UNEXP_TOKEN;
-      math_equation_set_parse_error(eq, token->ptr);
+      math_equation_set_parse_error(eq, token->ptr, str);
       math_scanner_free_token(token);
       math_expression_free(top);
       return NULL;
@@ -1113,7 +1114,7 @@ parse_expression_list(const char ** str, MathEquation *eq, int inside_block, int
 	goto End;
       }
       *err = MATH_ERROR_UNEXP_TOKEN;
-      math_equation_set_parse_error(eq, token->ptr);
+      math_equation_set_parse_error(eq, token->ptr, str);
       math_scanner_free_token(token);
       math_expression_free(top);
       return NULL;
@@ -1128,14 +1129,15 @@ parse_expression_list(const char ** str, MathEquation *eq, int inside_block, int
 MathExpression *
 math_parser_parse(const char *line, MathEquation *eq, int *err)
 {
-  const char *ptr = line;
   struct math_token *token, *tmp;
   MathExpression *exp;
+  struct math_string str;
 
   *err = MATH_ERROR_NONE;
 
+  math_scanner_init_string(&str, line);
   st_look_ahead_token = NULL;
-  exp = parse_expression_list(&ptr, eq, 0, err);
+  exp = parse_expression_list(&str, eq, 0, err);
 
   token = st_look_ahead_token;
   while (token) {
