@@ -1,11 +1,16 @@
-/* 
+/*
  * $Id: math_scanner.h,v 1.2 2009-11-10 04:12:20 hito Exp $
- * 
+ *
  */
 
 #ifndef MATH_SCANNER_HEADER
 #define MATH_SCANNER_HEADER
 
+struct math_string {
+  const char *top, *cur;
+  int line, ofst;
+};
+
 #include "math_operator.h"
 #include "math_constant.h"
 
@@ -41,7 +46,8 @@ struct math_token {
   struct math_token *next;
 };
 
-struct math_token *math_scanner_get_token(const char *str, const char ** rstr);
+struct math_token *math_scanner_get_token(struct math_string *rstr);
 void math_scanner_free_token(struct math_token *token);
+void math_scanner_init_string(struct math_string *str, const char *line);
 
 #endif
