@@ -30,6 +30,12 @@
 #include <glib.h>
 #include "mathfn.h"
 
+double
+distance(double x1, double y1)
+{
+  return sqrt(x1 * x1 + y1 * y1);
+}
+
 int
 compare_double(double x, double y)
 {
@@ -51,7 +57,7 @@ cutdown(double x)
 double
 nraise(double x)
 {
-  if (x<0) return (int )x;
+  if (x<0) return (int ) (x - N_EPSILON);
   else if ((x-(int )x)==0.0) return (int )x;
   else return (int )(x+1);
 }
