@@ -24,6 +24,7 @@
 #define MPI 3.1415926535897932385
 #define MEXP1 2.71828182845905
 #define MEULER 0.57721566490153286
+#define N_EPSILON 1E-12
 
 typedef double vector[11];
 typedef vector matrix[11];
@@ -36,3 +37,4 @@ int matsolv(int dim,matrix a,vector b,vector x);
 void HSB2RGB(double h,double s,double b,int *R,int *G,int *B);
 int bsearch_int(int *ary, int n, int val, int *idx);
 int compare_double(double x, double y);
+double distance(double x1, double y1);
