@@ -107,6 +107,23 @@ static char *anumalignchar[]={
   NULL
 };
 
+enum AXIS_NUM_NO_ZERO {
+  AXIS_NUM_NO_ZERO_REGULAR,
+  AXIS_NUM_NO_ZERO_NO_ZERO,
+  AXIS_NUM_NO_ZERO_NO_FLOATING_POINT,
+  AXIS_NUM_NO_ZERO_FALSE,
+  AXIS_NUM_NO_ZERO_TRUE,
+};
+
+static char *anumnozero[]={
+  N_("regular"),
+  N_("no_zero"),
+  N_("no_floating_point"),
+  "\0false",			/* for backward compatibility */
+  "\0true",			/* for backward compatibility */
+  NULL,
+};
+
 enum AXIS_NUM_ALIGN {
   AXIS_NUM_ALIGN_CENTER,
   AXIS_NUM_ALIGN_LEFT,
@@ -123,7 +140,7 @@ static char *anumdirchar[]={
   N_("oblique1"),
   N_("oblique2"),
   "\0normal",			/* for backward compatibility */
-  "\0parallel",			/* for backward compatibility */
+  "\0parallel",		/* for backward compatibility */
   NULL
 };
 
@@ -235,19 +252,19 @@ axisinit(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   char *font,*format,*group,*name;
 
   if (_exeparent(obj,(char *)argv[1],inst,rval,argc,argv)) return 1;
-  width=40;
+  width=DEFAULT_LINE_WIDTH;
   alen=72426;
   awid=60000;
   wlen=300;
-  wwid=40;
+  wwid=DEFAULT_LINE_WIDTH;
   len1=100;
-  wid1=40;
+  wid1=DEFAULT_LINE_WIDTH;
   len2=200;
-  wid2=40;
+  wid2=DEFAULT_LINE_WIDTH;
   len3=300;
-  wid3=40;
+  wid3=DEFAULT_LINE_WIDTH;
   bline=TRUE;
-  pt=2000;
+  pt=DEFAULT_FONT_PT;
   sx=0;
   sy=100;
   autonorm=5;
@@ -1416,16 +1433,18 @@ static int
 axiszoom2(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   int x,y,len,refx,refy,preserve_width;
-  double zoom;
-  int pt,space,wid1,wid2,wid3,len1,len2,len3,wid,wlen,wwid;
+  double zoom_x, zoom_y, zoom_n, zoom_p, zoom;
+  int pt,space,wid1,wid2,wid3,len1,len2,len3,wid,wlen,wwid,direction;
   struct narray *style,*gstyle;
   int i,snum,*sdata,gsnum,*gsdata;
 
   if (_exeparent(obj,(char *)argv[1],inst,rval,argc,argv)) return 1;
-  zoom=(*(int *)argv[2])/10000.0;
-  refx=(*(int *)argv[3]);
-  refy=(*(int *)argv[4]);
-  preserve_width = (*(int *)argv[5]);
+  zoom_x = (*(int *) argv[2]) / 10000.0;
+  zoom_y = (*(int *) argv[3]) / 10000.0;
+  zoom = MIN(zoom_x, zoom_y);
+  refx = (*(int *)argv[4]);
+  refy = (*(int *)argv[5]);
+  preserve_width = (*(int *)argv[6]);
   _getobj(obj,"x",inst,&x);
   _getobj(obj,"y",inst,&y);
   _getobj(obj,"length",inst,&len);
@@ -1442,17 +1461,19 @@ axiszoom2(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   _getobj(obj,"gauge_width3",inst,&wid3);
   _getobj(obj,"gauge_style",inst,&gstyle);
   _getobj(obj,"style",inst,&style);
+  _getobj(obj,"direction",inst,&direction);
   snum=arraynum(style);
   sdata=arraydata(style);
   gsnum=arraynum(gstyle);
   gsdata=arraydata(gstyle);
-  wlen*=zoom;
-  len1*=zoom;
-  len2*=zoom;
-  len3*=zoom;
-  x=(x-refx)*zoom+refx;
-  y=(y-refy)*zoom+refy;
-  len*=zoom;
+  direction = calc_zoom_direction(direction, zoom_x, zoom_y, &zoom_p, &zoom_n);
+  len*=zoom_p;
+  wlen*=zoom_n;
+  len1*=zoom_n;
+  len2*=zoom_n;
+  len3*=zoom_n;
+  x=(x-refx)*zoom_x+refx;
+  y=(y-refy)*zoom_y+refy;
   pt*=zoom;
   space*=zoom;
   if (! preserve_width) {
@@ -1478,6 +1499,7 @@ axiszoom2(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   if (_putobj(obj,"gauge_width2",inst,&wid2)) return 1;
   if (_putobj(obj,"gauge_length3",inst,&len3)) return 1;
   if (_putobj(obj,"gauge_width3",inst,&wid3)) return 1;
+  if (_putobj(obj,"direction",inst,&direction)) return 1;
 
   if (clear_bbox(obj, inst))
     return 1;
@@ -1933,13 +1955,17 @@ mjd_to_date_str(const struct axis_config *aconf, double mjd, const gchar *date_f
 }
 
 static char *
-get_axis_gauge_num_str(const char *format, double a)
+get_axis_gauge_num_str(const char *format, double a, int no_flt_zero)
 {
   int i, j, len;
   char *s;
   char format2[256], pm[] = "±";
   GString *num;
 
+  if (no_flt_zero && a == 0.0) {
+    return g_strdup("0");
+  }
+
   num = g_string_sized_new(16);
   if (num == NULL) {
     return NULL;
@@ -2024,7 +2050,7 @@ numformat(char **text, int *nlen, const char *format,
 	  const struct axis_config *aconf,
 	  const struct axislocal *alocal,
 	  int logpow, double po, double norm,
-	  const char *head, const char *tail, const char *date_format)
+	  const char *head, const char *tail, const char *date_format, int nozero)
 {
   int lpow;
   char *num;
@@ -2049,10 +2075,10 @@ numformat(char **text, int *nlen, const char *format,
     if (num) {
       logpow = 0;
     } else {
-      num = get_axis_gauge_num_str(format, a);
+      num = get_axis_gauge_num_str(format, a, nozero == AXIS_NUM_NO_ZERO_NO_FLOATING_POINT);
     }
   } else {
-    num = get_axis_gauge_num_str(format, a);
+    num = get_axis_gauge_num_str(format, a, nozero == AXIS_NUM_NO_ZERO_NO_FLOATING_POINT);
   }
 
   lpow = (logpow && (alocal->atype == AXISLOGBIG || alocal->atype == AXISLOGNORM));
@@ -2266,8 +2292,8 @@ draw_numbering(struct objlist *obj, N_VALUE *inst, struct axislocal *alocal,
       gy0=gy0-sy*cos(aconf->dir)-sx*sin(aconf->dir)+dly;
       if ((cstep==step) || ((alocal->atype==AXISLOGSMALL) && (rcode==3))) {
 	numcount++;
-	if (((numcount<=nnum) || (nnum==-1)) && ((po!=0) || !nozero)) {
-	  value = numformat(&text, &numlen, format, aconf, alocal, logpow, po, norm, head, tail, date_format);
+	if (((numcount<=nnum) || (nnum==-1)) && ((po!=0) || (nozero != AXIS_NUM_NO_ZERO_NO_ZERO))) {
+	  value = numformat(&text, &numlen, format, aconf, alocal, logpow, po, norm, head, tail, date_format, nozero);
 	  if (text == NULL) {
 	    return 1;
 	  }
@@ -2517,9 +2543,9 @@ numbering(struct objlist *obj, N_VALUE *inst, int GC, struct axis_config *aconf,
       numcount++;
 
       if ((numcount <= nnum || nnum == -1) &&
-	  (po != 0 || ! nozero)) {
+	  (po != 0 || (nozero != AXIS_NUM_NO_ZERO_NO_ZERO))) {
 
-	numformat(&text, &numlen, format, aconf, &alocal, logpow, po, norm, head, tail, date_format);
+	numformat(&text, &numlen, format, aconf, &alocal, logpow, po, norm, head, tail, date_format, nozero);
 	if (text == NULL) {
 	  return 1;
 	}
@@ -3865,6 +3891,28 @@ axisscalepop(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,
   return 0;
 }
 
+static int
+anumnozeroput(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
+{
+  int type;
+
+  type = * (int *) argv[2];
+  switch (type) {
+  case AXIS_NUM_NO_ZERO_REGULAR:
+  case AXIS_NUM_NO_ZERO_NO_ZERO:
+  case AXIS_NUM_NO_ZERO_NO_FLOATING_POINT:
+    break;
+  case AXIS_NUM_NO_ZERO_TRUE:
+    * (int *) argv[2] = AXIS_NUM_NO_ZERO_NO_ZERO;
+    break;
+  case AXIS_NUM_NO_ZERO_FALSE:
+    * (int *) argv[2] = AXIS_NUM_NO_ZERO_REGULAR;
+    break;
+  }
+
+  return 0;
+}
+
 static int
 anumdirput(struct objlist *obj,N_VALUE *inst,N_VALUE *rval, int argc,char **argv)
 {
@@ -3989,7 +4037,7 @@ static struct objtable axis[] = {
   {"num_font_style",NINT,NREAD|NWRITE,NULL,NULL,0},
   {"num_script_size",NINT,NREAD|NWRITE,axisput,NULL,0},
   {"num_align",NENUM,NREAD|NWRITE,NULL,anumalignchar,0},
-  {"num_no_zero",NBOOL,NREAD|NWRITE,NULL,NULL,0},
+  {"num_no_zero",NENUM,NREAD|NWRITE,anumnozeroput,anumnozero,0},
   {"num_direction",NENUM,NREAD|NWRITE,anumdirput,anumdirchar,0},
   {"num_shift_p",NINT,NREAD|NWRITE,NULL,NULL,0},
   {"num_shift_n",NINT,NREAD|NWRITE,NULL,NULL,0},
@@ -4013,7 +4061,7 @@ static struct objtable axis[] = {
   {"rotate",NVFUNC,NREAD|NEXEC,axisrotate,"iiii",0},
   {"flip",NVFUNC,NREAD|NEXEC,axisflip,"iii",0},
   {"change",NVFUNC,NREAD|NEXEC,axischange,"iii",0},
-  {"zooming",NVFUNC,NREAD|NEXEC,axiszoom,"iiii",0},
+  {"zooming",NVFUNC,NREAD|NEXEC,axiszoom,"iiiii",0},
   {"match",NBFUNC,NREAD|NEXEC,axismatch,"iiiii",0},
   {"coordinate",NDFUNC,NREAD|NEXEC,axiscoordinate,"ii",0},
   {"tight",NVFUNC,NREAD|NEXEC,axistight,"",0},
