@@ -10,6 +10,12 @@ enum DATA_OBJ_SOURCE {
   DATA_SOURCE_RANGE,
 };
 
+enum MOVING_AVERAGE_TYPE {
+  MOVING_AVERAGE_SIMPLE,
+  MOVING_AVERAGE_WEIGHTED,
+  MOVING_AVERAGE_EXPONENTIAL,
+};
+
 enum FILE_OBJ_PLOT_TYPE {
   PLOT_TYPE_MARK,
   PLOT_TYPE_LINE,
