@@ -49,8 +49,8 @@ markinit(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   int size,width,r2,g2,b2,a2;
 
   if (_exeparent(obj,(char *)argv[1],inst,rval,argc,argv)) return 1;
-  size=200;
-  width=40;
+  size=DEFAULT_MARK_SIZE;
+  width=DEFAULT_LINE_WIDTH;
   r2=255;
   g2=255;
   b2=255;
@@ -159,172 +159,6 @@ markmove(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   return 0;
 }
 
-#if MODIFY_MARK_TYPE
-
-static int
-rotate_cw(int mark) {
-  switch (mark) {
-  case 6:
-  case 16:
-  case 26:
-  case 74:
-    return mark + 3;
-  case 7:
-  case 17:
-  case 27:
-  case 72:
-  case 78:
-  case 75:
-    return mark + 1;
-  case 8:
-  case 9:
-  case 18:
-  case 19:
-  case 28:
-  case 29:
-  case 76:
-  case 77:
-    return mark - 2;
-  case 30:
-  case 31:
-  case 32:
-  case 33:
-  case 34:
-  case 35:
-  case 36:
-  case 37:
-    return mark + 30;
-  case 38:
-  case 39:
-  case 48:
-  case 49:
-    return mark + 20;
-  case 40:
-  case 41:
-  case 42:
-  case 43:
-  case 44:
-  case 45:
-  case 46:
-  case 47:
-    return mark + 10;
-  case 50:
-  case 51:
-  case 52:
-  case 53:
-  case 54:
-  case 55:
-  case 58:
-  case 59:
-  case 60:
-  case 61:
-  case 62:
-  case 63:
-  case 64:
-  case 65:
-    return mark - 20;
-  case 56:
-  case 66:
-  case 68:
-    return mark - 19;
-  case 57:
-  case 67:
-  case 69:
-    return mark - 21;
-  case 73:
-  case 79:
-    return mark - 1;
-  default:
-    return mark;
-  }
-}
-
-static int
-v_flip(int mark) {
-  switch (mark) {
-  case 8:
-  case 18:
-  case 28:
-  case 48:
-  case 56:
-  case 66:
-  case 74:
-    return mark + 1;
-  case 9:
-  case 19:
-  case 29:
-  case 49:
-  case 57:
-  case 67:
-  case 75:
-    return mark - 1;
-  case 30:
-  case 31:
-  case 32:
-  case 33:
-  case 34:
-  case 35:
-  case 36:
-  case 37:
-    return mark + 10;
-  case 40:
-  case 41:
-  case 42:
-  case 43:
-  case 44:
-  case 45:
-  case 46:
-  case 47:
-    return mark - 10;
-  default:
-    return mark;
-  }
-}
-
-static int
-h_flip(int mark) {
-  switch (mark) {
-  case 6:
-  case 16:
-  case 26:
-  case 36:
-  case 46:
-  case 68:
-  case 76:
-    return mark + 1;
-  case 7:
-  case 17:
-  case 27:
-  case 37:
-  case 47:
-  case 69:
-  case 77:
-    return mark - 1;
-  case 50:
-  case 51:
-  case 52:
-  case 53:
-  case 54:
-  case 55:
-  case 56:
-  case 57:
-    return mark + 10;
-  case 60:
-  case 61:
-  case 62:
-  case 63:
-  case 64:
-  case 65:
-  case 66:
-  case 67:
-    return mark - 10;
-  default:
-    return mark;
-  }
-}
-
-#endif	/* MODIFY_MARK_TYPE */
-
 static int
 markrotate(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
@@ -341,22 +175,8 @@ markrotate(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   angle = *(int *) argv[2];
 
 #if MODIFY_MARK_TYPE
-  angle %= 36000;
-  if (angle < 0)
-    angle += 36000;
-
   _getobj(obj, "type", inst, &type);
-
-  switch (angle) {
-  case 9000:
-    type = rotate_cw(type);
-  case 18000:
-    type = rotate_cw(type);
-  case 27000:
-    type = rotate_cw(type);
-    break;
-  }
-
+  type = mark_rotate(angle, type)
   _putobj(obj, "type", inst, &type);
 #endif
 
@@ -389,16 +209,7 @@ markflip(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 
 #if MODIFY_MARK_TYPE
   _getobj(obj, "type", inst, &type);
-
-  switch (dir) {
-  case FLIP_DIRECTION_VERTICAL:
-    type = v_flip(type);
-    break;
-  case FLIP_DIRECTION_HORIZONTAL:
-    type = h_flip(type);
-    break;
-  }
-
+  type = mark_flip(dir, type);
   _putobj(obj, "type", inst, &type);
 #endif
 
@@ -424,14 +235,16 @@ static int
 markzoom(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   int i,x,y,size,refx,refy,width,snum,*sdata,preserve_width;
-  double zoom;
+  double zoom, zoom_x, zoom_y;
   struct narray *style;
 
   if (_exeparent(obj,(char *)argv[1],inst,rval,argc,argv)) return 1;
-  zoom=(*(int *)argv[2])/10000.0;
-  refx=(*(int *)argv[3]);
-  refy=(*(int *)argv[4]);
-  preserve_width = (*(int *)argv[5]);
+  zoom_x = (*(int *) argv[2]) / 10000.0;
+  zoom_y = (*(int *) argv[3]) / 10000.0;
+  zoom = MIN(zoom_x, zoom_y);
+  refx = (*(int *)argv[4]);
+  refy = (*(int *)argv[5]);
+  preserve_width = (*(int *)argv[6]);
   _getobj(obj,"x",inst,&x);
   _getobj(obj,"y",inst,&y);
   _getobj(obj,"size",inst,&size);
@@ -439,8 +252,8 @@ markzoom(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   _getobj(obj,"style",inst,&style);
   snum=arraynum(style);
   sdata=arraydata(style);
-  x=(x-refx)*zoom+refx;
-  y=(y-refy)*zoom+refy;
+  x=(x-refx)*zoom_x+refx;
+  y=(y-refy)*zoom_y+refy;
   size=size*zoom;
   if (! preserve_width) {
     width=width*zoom;
@@ -549,7 +362,7 @@ static struct objtable mark[] = {
   {"move",NVFUNC,NREAD|NEXEC,markmove,"ii",0},
   {"rotate",NVFUNC,NREAD|NEXEC,markrotate,"iiii",0},
   {"flip",NVFUNC,NREAD|NEXEC,markflip,"iii",0},
-  {"zooming",NVFUNC,NREAD|NEXEC,markzoom,"iiii",0},
+  {"zooming",NVFUNC,NREAD|NEXEC,markzoom,"iiiii",0},
   {"match",NBFUNC,NREAD|NEXEC,markmatch,"iiiii",0},
 
   {"hsb",NVFUNC,NREAD|NEXEC,put_hsb,"ddd",0},
