@@ -113,7 +113,7 @@ arrowinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
     type = PATH_TYPE_LINE;
   }
 
-  width = 40;
+  width = DEFAULT_LINE_WIDTH;
   headlen = 72426;
   headwidth = 60000;
   miter = 1000;
@@ -151,9 +151,6 @@ arrowdone(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
   return 0;
 }
 
-#define ARROW_SIZE_MIN 10000
-#define ARROW_SIZE_MAX 200000
-
 static int
 arrowput(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
@@ -215,9 +212,15 @@ curve_clear(struct objlist *obj,N_VALUE *inst)
 }
 
 static double
-distance(double x1, double y1)
+get_dx_dy(int x0, int y0, int x1, int y1, double *dx, double *dy)
 {
-  return sqrt(x1 * x1 + y1 * y1);
+  double len, x, y;
+  x = x0 - x1;
+  y = y0 - y1;
+  len = distance(x, y);
+  *dx = x / len;
+  *dy = y / len;
+  return len;
 }
 
 static void
@@ -231,11 +234,7 @@ get_arrow_pos(int *points2, int n,
   alen = width * (double) headlen / 10000;
   awidth = width * (double) headwidth / 20000;
 
-  dx = x0 - x1;
-  dy = y0 - y1;
-  len = distance(dx, dy);
-  dx /= len;
-  dy /= len;
+  len = get_dx_dy(x0, y0, x1, y1, &dx, &dy);
   ax0 = nround(x0 - dx * alen);
   ay0 = nround(y0 - dy * alen);
   alen2 = alen * width / awidth / 2;
@@ -261,12 +260,37 @@ get_arrow_pos(int *points2, int n,
   ap[5] = nround(ay0 - dx * awidth);
 }
 
+static void
+draw_marker(struct objlist *obj, N_VALUE *inst, int GC, int type, int mark_type, int *ap, int join, int miter,
+	    int x0, int y0, int x1, int y1, int width, int fr, int fg, int fb, int fa, int headlen, int headwidth)
+{
+  double dx, dy;
+  switch (type) {
+  case MARKER_TYPE_ARROW:
+    GRAlinestyle(GC, 0, NULL, 1, GRA_LINE_CAP_BUTT, join, miter);
+    GRAdrawpoly(GC, 3, ap, GRA_FILL_MODE_EVEN_ODD);
+    break;
+  case MARKER_TYPE_WAVE:
+    get_dx_dy(x0, y0, x1, y1, &dx, &dy);
+    draw_marker_wave(obj, inst, GC, width, headlen, headwidth, x0, y0, dx, dy, ERRSPL);
+    break;
+  case MARKER_TYPE_MARK:
+    get_dx_dy(x0, y0, x1, y1, &dx, &dy);
+    draw_marker_mark(obj, inst, GC, width, headlen, headwidth, x0, y0, dx, dy, fr, fg, fb, fa, mark_type);
+    break;
+  case MARKER_TYPE_BAR:
+    get_dx_dy(x0, y0, x1, y1, &dx, &dy);
+    draw_marker_bar(obj, inst, GC, width, headlen, headwidth, x0, y0, dx, dy);
+    break;
+  }
+}
+
 static void
 draw_stroke(struct objlist *obj, N_VALUE *inst, int GC, int *points2, int *pdata, int num, int close_path)
 {
   int width, fr, fg, fb, fa, headlen, headwidth;
-  int join, miter, head;
-  int x, y, x0, y0, x1, y1, x2, y2, x3, y3;
+  int join, miter, head_begin, head_end;
+  int x, y, x0, y0, x1, y1, x2, y2, x3, y3, type;
   struct narray *style;
   int snum, *sdata;
   int i;
@@ -281,7 +305,9 @@ draw_stroke(struct objlist *obj, N_VALUE *inst, int GC, int *points2, int *pdata
   _getobj(obj, "style",        inst, &style);
   _getobj(obj, "join",         inst, &join);
   _getobj(obj, "miter_limit",  inst, &miter);
-  _getobj(obj, "arrow",        inst, &head);
+  _getobj(obj, "marker_begin",  inst, &head_begin);
+  _getobj(obj, "marker_end",    inst, &head_end);
+
   _getobj(obj, "arrow_length", inst, &headlen);
   _getobj(obj, "arrow_width",  inst, &headwidth);
 
@@ -300,13 +326,13 @@ draw_stroke(struct objlist *obj, N_VALUE *inst, int GC, int *points2, int *pdata
   x3 = points2[2 * num - 2];
   y3 = points2[2 * num - 1];
 
-  if (head == ARROW_POSITION_BEGIN || head == ARROW_POSITION_BOTH) {
+  if (head_begin == MARKER_TYPE_ARROW) {
     get_arrow_pos(points2, 0,
 		  width, headlen, headwidth,
 		  x0, y0, x1, y1, ap);
   }
 
-  if (head == ARROW_POSITION_END || head == ARROW_POSITION_BOTH) {
+  if (head_end == MARKER_TYPE_ARROW) {
     get_arrow_pos(points2, num * 2 - 2,
 		  width, headlen, headwidth,
 		  x3, y3, x2, y2, ap2);
@@ -326,15 +352,10 @@ draw_stroke(struct objlist *obj, N_VALUE *inst, int GC, int *points2, int *pdata
     }
   }
 
-  if (head == ARROW_POSITION_BEGIN || head == ARROW_POSITION_BOTH) {
-    GRAlinestyle(GC, 0, NULL, 1, GRA_LINE_CAP_BUTT, join, miter);
-    GRAdrawpoly(GC, 3, ap, GRA_FILL_MODE_EVEN_ODD);
-  }
-
-  if (head == ARROW_POSITION_END || head == ARROW_POSITION_BOTH) {
-    GRAlinestyle(GC, 0, NULL, 1, GRA_LINE_CAP_BUTT, join, miter);
-    GRAdrawpoly(GC, 3, ap2, GRA_FILL_MODE_EVEN_ODD);
-  }
+  _getobj(obj, "mark_type_begin", inst, &type);
+  draw_marker(obj, inst, GC, head_begin, type, ap, join, miter, x0, y0, x1, y1, width, fr, fg, fb, fa, headlen, headwidth);
+  _getobj(obj, "mark_type_end", inst, &type);
+  draw_marker(obj, inst, GC, head_end, type, ap2, join, miter, x3, y3, x2, y2, width, fr, fg, fb, fa, headlen, headwidth);
 }
 
 static void
@@ -445,10 +466,11 @@ arrowbbox(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
   struct narray *array;
   int i, j, width;
   int headlen, headwidth;
-  int head;
+  int head_begin, head_end;
   int *points2;
   int x0, y0, x1, y1, x2, y2, x3, y3;
   int ap[6], ap2[6];
+  double dx, dy, awidth, w;
 
   array = rval->array;
   if (arraynum(array) != 0) {
@@ -474,10 +496,12 @@ arrowbbox(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
   }
 
   _getobj(obj, "width",        inst, &width);
-  _getobj(obj, "arrow",        inst, &head);
+  _getobj(obj, "marker_begin",  inst, &head_begin);
+  _getobj(obj, "marker_end",    inst, &head_end);
   _getobj(obj, "arrow_length", inst, &headlen);
   _getobj(obj, "arrow_width",  inst, &headwidth);
 
+  awidth = width * (double) headwidth / 10000;
   num = arraynum(points) / 2;
   pdata = arraydata(points);
   points2 = g_malloc(sizeof(int) * num * 2);
@@ -513,16 +537,64 @@ arrowbbox(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
   y3 = points2[2 * num2 - 1];
   g_free(points2);
 
-  if (head==ARROW_POSITION_BEGIN || head==ARROW_POSITION_BOTH) {
+  switch (head_begin) {
+  case MARKER_TYPE_ARROW:
     get_arrow_pos(NULL, 0,
 		  width, headlen, headwidth,
 		  x0, y0, x1, y1, ap);
+    break;
+  case MARKER_TYPE_WAVE:
+  case MARKER_TYPE_BAR:
+    ap[0] = x0;
+    ap[1] = y0 - awidth;
+    ap[2] = x0 + 0.25 * awidth;
+    ap[3] = y0 + 0.5 * awidth;
+    ap[4] = x0;
+    ap[5] = y0 + awidth;
+    get_dx_dy(x0, y0, x1, y1, &dx, &dy);
+    GRArotate(x0, y0, ap, ap, 3, dx, dy);
+    break;
+  case MARKER_TYPE_MARK:
+    w = awidth / 2;
+    ap[0] = x0 + w;
+    ap[1] = y0 + w;
+    ap[2] = x0 + w;
+    ap[3] = y0 - w;
+    ap[4] = x0;
+    ap[5] = y0;
+    get_dx_dy(x0, y0, x1, y1, &dx, &dy);
+    GRArotate(x0, y0, ap, ap, 2, dx, dy);
+    break;
   }
 
-  if (head==ARROW_POSITION_END || head==ARROW_POSITION_BOTH) {
+  switch (head_end) {
+  case MARKER_TYPE_ARROW:
     get_arrow_pos(NULL, 0,
 		  width, headlen, headwidth,
 		  x3, y3, x2, y2, ap2);
+    break;
+  case MARKER_TYPE_WAVE:
+  case MARKER_TYPE_BAR:
+    ap2[0] = x3;
+    ap2[1] = y3 - awidth;
+    ap2[2] = x3 + 0.25 * awidth;
+    ap2[3] = y3 + 0.5 * awidth;
+    ap2[4] = x3;
+    ap2[5] = y3 + awidth;
+    get_dx_dy(x3, y3, x2, y2, &dx, &dy);
+    GRArotate(x3, y3, ap2, ap2, 3, dx, dy);
+    break;
+  case MARKER_TYPE_MARK:
+    w = awidth / 2;
+    ap2[0] = x3 + w;
+    ap2[1] = y3 + w;
+    ap2[2] = x3 + w;
+    ap2[3] = y3 - w;
+    ap2[4] = x3;
+    ap2[5] = y3;
+    get_dx_dy(x3, y3, x2, y2, &dx, &dy);
+    GRArotate(x3, y3, ap2, ap2, 2, dx, dy);
+    break;
   }
   if (array == NULL && (array = arraynew(sizeof(int))) == NULL) {
     return 1;
@@ -555,7 +627,7 @@ arrowbbox(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
   }
 
   if (stroke) {
-    if (head == ARROW_POSITION_BEGIN || head == ARROW_POSITION_BOTH) {
+    if (head_begin != MARKER_TYPE_NONE) {
       for (i = 0; i < 3; i++) {
 	if (ap[i * 2] < minx) minx = ap[i * 2];
 	if (ap[i * 2] > maxx) maxx = ap[i * 2];
@@ -563,7 +635,7 @@ arrowbbox(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
 	if (ap[i * 2 + 1] > maxy) maxy = ap[i * 2 + 1];
       }
     }
-    if (head == ARROW_POSITION_END || head == ARROW_POSITION_BOTH) {
+    if (head_end != MARKER_TYPE_NONE) {
       for (i = 0; i < 3 ; i++) {
 	if (ap2[i * 2] < minx) minx = ap2[i * 2];
 	if (ap2[i * 2] > maxx) maxx = ap2[i * 2];
@@ -746,8 +818,21 @@ curvematch(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **a
 static int
 curve_flip(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
-  curve_clear(obj, inst);
+#if ! ROTATE_MARK
+  int type_begin, type_end;
+  enum FLIP_DIRECTION dir;
 
+  dir = (* (int *) argv[2] == FLIP_DIRECTION_HORIZONTAL) ? FLIP_DIRECTION_HORIZONTAL : FLIP_DIRECTION_VERTICAL;
+
+  _getobj(obj, "mark_type_begin", inst, &type_begin);
+  _getobj(obj, "mark_type_end", inst, &type_end);
+  type_begin = mark_flip(dir, type_begin);
+  type_end = mark_flip(dir, type_end);
+  _putobj(obj, "mark_type_begin", inst, &type_begin);
+  _putobj(obj, "mark_type_end", inst, &type_end);
+#endif
+
+  curve_clear(obj, inst);
   return legendflip(obj, inst, rval, argc, argv);
 }
 
@@ -777,6 +862,18 @@ curve_move(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **a
 static int
 curve_rotate(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
+#if ! ROTATE_MARK
+  int type_begin, type_end, angle;
+
+  angle = *(int *) argv[2];
+
+  _getobj(obj, "mark_type_begin", inst, &type_begin);
+  _getobj(obj, "mark_type_end", inst, &type_end);
+  type_begin = mark_rotate(angle, type_begin);
+  type_end = mark_rotate(angle, type_end);
+  _putobj(obj, "mark_type_begin", inst, &type_begin);
+  _putobj(obj, "mark_type_end", inst, &type_end);
+#endif
   curve_clear(obj, inst);
 
   return legendrotate(obj, inst, rval, argc, argv);
@@ -829,6 +926,42 @@ put_fill_mode(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char
   return legendgeometry(obj, inst, rval, argc, argv);
 }
 
+static int
+arrowput_arrow(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
+{
+  int arrow, type;
+
+  arrow = * (int *) argv[2];
+  switch (arrow) {
+  case ARROW_POSITION_NONE:
+    type = MARKER_TYPE_NONE;
+    _putobj(obj, "marker_begin", inst, &type);
+    _putobj(obj, "marker_end", inst, &type);
+    break;
+  case ARROW_POSITION_END:
+    type = MARKER_TYPE_NONE;
+    _putobj(obj, "marker_begin", inst, &type);
+    type = MARKER_TYPE_ARROW;
+    _putobj(obj, "marker_end", inst, &type);
+    break;
+  case ARROW_POSITION_BEGIN:
+    type = MARKER_TYPE_ARROW;
+    _putobj(obj, "marker_begin", inst, &type);
+    type = MARKER_TYPE_NONE;
+    _putobj(obj, "marker_end", inst, &type);
+    break;
+  case ARROW_POSITION_BOTH:
+    type = MARKER_TYPE_ARROW;
+    _putobj(obj, "marker_begin", inst, &type);
+    _putobj(obj, "marker_end", inst, &type);
+    break;
+  }
+  if (clear_bbox(obj, inst)) {
+    return 1;
+  }
+  return 0;
+}
+
 static struct objtable arrow[] = {
   {"init", NVFUNC, NEXEC, arrowinit, NULL, 0},
   {"done", NVFUNC, NEXEC, arrowdone, NULL, 0},
@@ -860,9 +993,12 @@ static struct objtable arrow[] = {
   {"join", NENUM, NREAD|NWRITE, NULL, joinchar, 0},
   {"miter_limit", NINT, NREAD|NWRITE, oputge1, NULL, 0},
 
-  {"arrow", NENUM, NREAD|NWRITE, arrowput, arrowchar, 0},
+  {"marker_begin", NENUM, NREAD|NWRITE, arrowput, marker_type_char, 0},
+  {"marker_end", NENUM, NREAD|NWRITE, arrowput, marker_type_char, 0},
   {"arrow_length", NINT, NREAD|NWRITE, arrowput, NULL, 0},
   {"arrow_width", NINT, NREAD|NWRITE, arrowput, NULL, 0},
+  {"mark_type_begin",NINT,NREAD|NWRITE,oputmarktype,NULL,0},
+  {"mark_type_end",NINT,NREAD|NWRITE,oputmarktype,NULL,0},
 
   {"draw", NVFUNC, NREAD|NEXEC, arrowdraw, "i", 0},
   {"bbox", NIAFUNC, NREAD|NEXEC, arrowbbox, "", 0},
@@ -881,6 +1017,7 @@ static struct objtable arrow[] = {
   {"G", NINT, NWRITE, put_color_for_backward_compatibility, NULL, 0},
   {"B", NINT, NWRITE, put_color_for_backward_compatibility, NULL, 0},
   {"A", NINT, NWRITE, put_color_for_backward_compatibility, NULL, 0},
+  {"arrow", NENUM, NWRITE, arrowput_arrow, arrowchar, 0},
 };
 
 #define TBLNUM (sizeof(arrow) / sizeof(*arrow))
@@ -891,4 +1028,3 @@ addpath(void)
 {
   return addobject(NAME, ALIAS, PARENT, OVERSION, TBLNUM, arrow, ERRNUM, patherrorlist, NULL, NULL);
 }
-
